@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2021 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -58,6 +58,7 @@ import weakref
 import pyglet
 from pyglet import gl
 
+
 if not getattr(sys, 'is_pyglet_doc_run', False):
     if pyglet.compat_platform == 'darwin':
         from pyglet.font.quartz import QuartzFont
@@ -65,13 +66,9 @@ if not getattr(sys, 'is_pyglet_doc_run', False):
 
     elif pyglet.compat_platform in ('win32', 'cygwin'):
         from pyglet.libs.win32.constants import WINDOWS_7_OR_GREATER
-        if WINDOWS_7_OR_GREATER:
-            if pyglet.options["advanced_font_features"] is True:
-                from pyglet.font.directwrite import Win32DirectWriteFont
-                _font_class = Win32DirectWriteFont
-            else:
-                from pyglet.font.win32 import GDIPlusFont
-                _font_class = GDIPlusFont
+        if WINDOWS_7_OR_GREATER and not pyglet.options['win32_gdi_font']:
+            from pyglet.font.directwrite import Win32DirectWriteFont
+            _font_class = Win32DirectWriteFont
         else:
             from pyglet.font.win32 import GDIPlusFont
             _font_class = GDIPlusFont
@@ -97,8 +94,7 @@ def load(name=None, size=None, bold=False, italic=False, stretch=False, dpi=None
             pyglet 1.1, the name may be omitted.
         `size` : float
             Size of the font, in points.  The returned font may be an exact
-            match or the closest available.  In pyglet 1.1, the size may be
-            omitted, and defaults to 12pt.
+            match or the closest available.
         `bold` : bool
             If True, a bold variant is returned, if one exists for the given
             family and size.
@@ -118,22 +114,34 @@ def load(name=None, size=None, bold=False, italic=False, stretch=False, dpi=None
     if dpi is None:
         dpi = 96
 
-    # Find first matching name
-    if type(name) in (tuple, list):
-        for n in name:
-            if _font_class.have_font(n):
-                name = n
-                break
-        else:
-            name = None
-
     # Locate or create font cache
     shared_object_space = gl.current_context.object_space
     if not hasattr(shared_object_space, 'pyglet_font_font_cache'):
         shared_object_space.pyglet_font_font_cache = weakref.WeakValueDictionary()
         shared_object_space.pyglet_font_font_hold = []
+        shared_object_space.pyglet_font_font_name_match = {}  # Match a tuple to specific name to reduce lookups.
+
     font_cache = shared_object_space.pyglet_font_font_cache
     font_hold = shared_object_space.pyglet_font_font_hold
+    font_name_match = shared_object_space.pyglet_font_font_name_match
+
+    name_type = type(name)
+    if name_type in (tuple, list):
+        if name_type == list:
+            name = tuple(name)
+
+        if name in font_name_match:
+            name = font_name_match[name]
+        else:
+            # Find first matching name, cache it.
+            found_name = None
+            for n in name:
+                if _font_class.have_font(n):
+                    found_name = n
+                    break
+
+            font_name_match[name] = found_name
+            name = found_name
 
     # Look for font name in font cache
     descriptor = (name, size, bold, italic, stretch, dpi)
