@@ -2,13 +2,14 @@ import copy
 
 import pyglet
 from pyglet.font import base
+from pyglet.util import debug_print
 from pyglet.image.codecs.wic import IWICBitmap, GUID_WICPixelFormat32bppBGR, WICDecoder, GUID_WICPixelFormat32bppBGRA, \
     GUID_WICPixelFormat32bppPBGRA
 
 from pyglet import image
 import ctypes
 import math
-from pyglet import com
+from pyglet.libs.win32 import com
 from pyglet.libs.win32 import _kernel32 as kernel32
 from pyglet.libs.win32 import _ole32 as ole32
 from pyglet.libs.win32.constants import *
@@ -32,6 +33,8 @@ except OSError as err:
     # Doesn't exist? Should stop import of library.
     pass
 
+_debug_font = debug_print('debug_font')
+
 
 def DWRITE_MAKE_OPENTYPE_TAG(a, b, c, d):
     return ord(d) << 24 | ord(c) << 16 | ord(b) << 8 | ord(a)
@@ -101,6 +104,32 @@ name_to_stretch = {"undefined": DWRITE_FONT_STRETCH_UNDEFINED,
                    "extraexpanded": DWRITE_FONT_STRETCH_EXTRA_EXPANDED,
                    }
 
+DWRITE_GLYPH_IMAGE_FORMATS = c_int
+
+DWRITE_GLYPH_IMAGE_FORMATS_NONE = 0x00000000
+DWRITE_GLYPH_IMAGE_FORMATS_TRUETYPE = 0x00000001
+DWRITE_GLYPH_IMAGE_FORMATS_CFF = 0x00000002
+DWRITE_GLYPH_IMAGE_FORMATS_COLR = 0x00000004
+DWRITE_GLYPH_IMAGE_FORMATS_SVG = 0x00000008
+DWRITE_GLYPH_IMAGE_FORMATS_PNG = 0x00000010
+DWRITE_GLYPH_IMAGE_FORMATS_JPEG = 0x00000020
+DWRITE_GLYPH_IMAGE_FORMATS_TIFF = 0x00000040
+DWRITE_GLYPH_IMAGE_FORMATS_PREMULTIPLIED_B8G8R8A8 = 0x00000080
+
+DWRITE_MEASURING_MODE = UINT
+DWRITE_MEASURING_MODE_NATURAL = 0
+DWRITE_MEASURING_MODE_GDI_CLASSIC = 1
+DWRITE_MEASURING_MODE_GDI_NATURAL = 2
+
+DWRITE_GLYPH_IMAGE_FORMATS_ALL = DWRITE_GLYPH_IMAGE_FORMATS_TRUETYPE | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_CFF | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_COLR | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_SVG | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_PNG | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_JPEG | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_TIFF | \
+                                 DWRITE_GLYPH_IMAGE_FORMATS_PREMULTIPLIED_B8G8R8A8
+
 DWRITE_FONT_STYLE = UINT
 DWRITE_FONT_STYLE_NORMAL = 0
 DWRITE_FONT_STYLE_OBLIQUE = 1
@@ -118,6 +147,31 @@ UINT32 = c_uint32
 UINT64 = c_uint64
 
 
+class D2D_POINT_2F(Structure):
+    _fields_ = (
+        ('x', FLOAT),
+        ('y', FLOAT),
+    )
+
+
+class D2D1_RECT_F(Structure):
+    _fields_ = (
+        ('left', FLOAT),
+        ('top', FLOAT),
+        ('right', FLOAT),
+        ('bottom', FLOAT),
+    )
+
+
+class D2D1_COLOR_F(Structure):
+    _fields_ = (
+        ('r', FLOAT),
+        ('g', FLOAT),
+        ('b', FLOAT),
+        ('a', FLOAT),
+    )
+
+
 class DWRITE_TEXT_METRICS(ctypes.Structure):
     _fields_ = (
         ('left', FLOAT),
@@ -197,7 +251,7 @@ class IDWriteFontFace(com.pIUnknown):
         ('GetMetrics',
          com.METHOD(c_void, POINTER(DWRITE_FONT_METRICS))),
         ('GetGlyphCount',
-         com.STDMETHOD()),
+         com.METHOD(UINT16)),
         ('GetDesignGlyphMetrics',
          com.STDMETHOD(POINTER(UINT16), UINT32, POINTER(DWRITE_GLYPH_METRICS), BOOL)),
         ('GetGlyphIndices',
@@ -249,6 +303,19 @@ class IDWriteFontFace1(IDWriteFontFace, com.pIUnknown):
     ]
 
 
+class DWRITE_GLYPH_RUN(ctypes.Structure):
+    _fields_ = (
+        ('fontFace', IDWriteFontFace),
+        ('fontEmSize', FLOAT),
+        ('glyphCount', UINT32),
+        ('glyphIndices', POINTER(UINT16)),
+        ('glyphAdvances', POINTER(FLOAT)),
+        ('glyphOffsets', POINTER(DWRITE_GLYPH_OFFSET)),
+        ('isSideways', BOOL),
+        ('bidiLevel', UINT32),
+    )
+
+
 DWRITE_SCRIPT_SHAPES = UINT
 DWRITE_SCRIPT_SHAPES_DEFAULT = 0
 
@@ -369,6 +436,7 @@ class TextAnalysis(com.COMObject):
         self._textlength = text_length
         self._glyphstart = 0
         self._glyphcount = 0
+        self._ptrs.clear()
 
         self._start_run = Run()
         self._start_run.text_length = text_length
@@ -395,7 +463,7 @@ class TextAnalysis(com.COMObject):
         return 0
         # return 0x80004001
 
-    def GetTextBeforePosition(self, this):
+    def GetTextBeforePosition(self, this, textPosition, textString, textLength):
         raise Exception("Currently not implemented.")
 
     def GetTextAtPosition(self, this, textPosition, textString, textLength):
@@ -405,6 +473,7 @@ class TextAnalysis(com.COMObject):
         # textPosition - The index of the first character of the text to retrieve.
         # textString - The pointer to the first character of text at the index requested.
         # textLength - The characters available at/after the textString pointer (string length).
+
         if textPosition >= self._textlength:
             self._no_ptr = c_wchar_p(None)
             textString[0] = self._no_ptr
@@ -658,9 +727,9 @@ class IDWriteTextFormat(com.pIUnknown):
         ('GetFontCollection',
          com.STDMETHOD()),
         ('GetFontFamilyNameLength',
-         com.STDMETHOD()),
+         com.STDMETHOD(UINT32, POINTER(UINT32))),
         ('GetFontFamilyName',
-         com.STDMETHOD()),
+         com.STDMETHOD(UINT32, c_wchar_p, UINT32)),
         ('GetFontWeight',
          com.STDMETHOD()),
         ('GetFontStyle',
@@ -740,9 +809,9 @@ class IDWriteTextLayout(IDWriteTextFormat, com.pIUnknown):
         ('GetFontCollection2',
          com.STDMETHOD()),
         ('GetFontFamilyNameLength2',
-         com.STDMETHOD()),
+         com.STDMETHOD(UINT32, POINTER(UINT32), c_void_p)),
         ('GetFontFamilyName2',
-         com.STDMETHOD()),
+         com.STDMETHOD(UINT32, c_wchar_p, UINT32, c_void_p)),
         ('GetFontWeight2',
          com.STDMETHOD(UINT32, POINTER(DWRITE_FONT_WEIGHT), POINTER(DWRITE_TEXT_RANGE))),
         ('GetFontStyle2',
@@ -992,6 +1061,21 @@ class LegacyCollectionLoader(com.COMObject):
 IID_IDWriteFactory = com.GUID(0xb859ee5a, 0xd838, 0x4b5b, 0xa2, 0xe8, 0x1a, 0xdc, 0x7d, 0x93, 0xdb, 0x48)
 
 
+class IDWriteRenderingParams(com.pIUnknown):
+    _methods_ = [
+        ('GetGamma',
+         com.METHOD(FLOAT)),
+        ('GetEnhancedContrast',
+         com.METHOD(FLOAT)),
+        ('GetClearTypeLevel',
+         com.METHOD(FLOAT)),
+        ('GetPixelGeometry',
+         com.METHOD(UINT)),
+        ('GetRenderingMode',
+         com.METHOD(UINT)),
+    ]
+
+
 class IDWriteFactory(com.pIUnknown):
     _methods_ = [
         ('GetSystemFontCollection',
@@ -1009,11 +1093,11 @@ class IDWriteFactory(com.pIUnknown):
         ('CreateFontFace',
          com.STDMETHOD()),
         ('CreateRenderingParams',
-         com.STDMETHOD()),
+         com.STDMETHOD(POINTER(IDWriteRenderingParams))),
         ('CreateMonitorRenderingParams',
          com.STDMETHOD()),
         ('CreateCustomRenderingParams',
-         com.STDMETHOD()),
+         com.STDMETHOD(FLOAT, FLOAT, FLOAT, UINT, UINT, POINTER(IDWriteRenderingParams))),
         ('RegisterFontFileLoader',
          com.STDMETHOD(c_void_p)),  # Ambigious as newer is a pIUnknown and legacy is IUnknown.
         ('UnregisterFontFileLoader',
@@ -1047,7 +1131,7 @@ class IDWriteFactory1(IDWriteFactory, com.pIUnknown):
     _methods_ = [
         ('GetEudcFontCollection',
          com.STDMETHOD()),
-        ('CreateCustomRenderingParams',
+        ('CreateCustomRenderingParams1',
          com.STDMETHOD()),
     ]
 
@@ -1055,7 +1139,7 @@ class IDWriteFactory1(IDWriteFactory, com.pIUnknown):
 class IDWriteFontFallback(com.pIUnknown):
     _methods_ = [
         ('MapCharacters',
-         com.STDMETHOD(POINTER(IDWriteTextAnalysisSource), UINT32, UINT32, POINTER(IDWriteFontCollection), c_wchar_p,
+         com.STDMETHOD(POINTER(IDWriteTextAnalysisSource), UINT32, UINT32, IDWriteFontCollection, c_wchar_p,
                        DWRITE_FONT_WEIGHT, DWRITE_FONT_STYLE, DWRITE_FONT_STRETCH, POINTER(UINT32),
                        POINTER(IDWriteFont),
                        POINTER(FLOAT))),
@@ -1070,7 +1154,7 @@ class IDWriteFactory2(IDWriteFactory1, com.pIUnknown):
          com.STDMETHOD()),
         ('TranslateColorGlyphRun',
          com.STDMETHOD()),
-        ('CreateCustomRenderingParams',
+        ('CreateCustomRenderingParams2',
          com.STDMETHOD()),
         ('CreateGlyphRunAnalysis',
          com.STDMETHOD()),
@@ -1122,7 +1206,7 @@ class IDWriteFactory3(IDWriteFactory2, com.pIUnknown):
     _methods_ = [
         ('CreateGlyphRunAnalysis',
          com.STDMETHOD()),
-        ('CreateCustomRenderingParams',
+        ('CreateCustomRenderingParams3',
          com.STDMETHOD()),
         ('CreateFontFaceReference',
          com.STDMETHOD()),
@@ -1138,14 +1222,26 @@ class IDWriteFactory3(IDWriteFactory2, com.pIUnknown):
          com.STDMETHOD()),
         ('GetFontDownloadQueue',
          com.STDMETHOD()),
-        ('GetSystemFontSet',
+        # ('GetSystemFontSet',
+        # com.STDMETHOD()),
+    ]
+
+
+class IDWriteColorGlyphRunEnumerator1(com.pIUnknown):
+    _methods_ = [
+        ('MoveNext',
+         com.STDMETHOD()),
+        ('GetCurrentRun',
          com.STDMETHOD()),
     ]
 
 
 class IDWriteFactory4(IDWriteFactory3, com.pIUnknown):
     _methods_ = [
-        ('TranslateColorGlyphRun',
+        ('TranslateColorGlyphRun4',  # Renamed to prevent clash from previous factories.
+         com.STDMETHOD(D2D_POINT_2F, DWRITE_GLYPH_RUN, c_void_p, DWRITE_GLYPH_IMAGE_FORMATS, DWRITE_MEASURING_MODE,
+                       c_void_p, UINT32, POINTER(IDWriteColorGlyphRunEnumerator1))),
+        ('ComputeGlyphOrigins_',
          com.STDMETHOD()),
         ('ComputeGlyphOrigins',
          com.STDMETHOD()),
@@ -1180,49 +1276,11 @@ class IDWriteFactory5(IDWriteFactory4, IDWriteFactory3, IDWriteFactory2, IDWrite
     ]
 
 
-class DWRITE_GLYPH_RUN(ctypes.Structure):
-    _fields_ = (
-        ('fontFace', IDWriteFontFace),
-        ('fontEmSize', FLOAT),
-        ('glyphCount', UINT32),
-        ('glyphIndices', POINTER(UINT16)),
-        ('glyphAdvances', POINTER(FLOAT)),
-        ('glyphOffsets', POINTER(DWRITE_GLYPH_OFFSET)),
-        ('isSideways', BOOL),
-        ('bidiLevel', UINT32),
-    )
-
-
 DWriteCreateFactory = dwrite_lib.DWriteCreateFactory
 DWriteCreateFactory.restype = HRESULT
 DWriteCreateFactory.argtypes = [DWRITE_FACTORY_TYPE, com.REFIID, POINTER(com.pIUnknown)]
 
 
-class D2D_POINT_2F(Structure):
-    _fields_ = (
-        ('x', FLOAT),
-        ('y', FLOAT),
-    )
-
-
-class D2D1_RECT_F(Structure):
-    _fields_ = (
-        ('left', FLOAT),
-        ('top', FLOAT),
-        ('right', FLOAT),
-        ('bottom', FLOAT),
-    )
-
-
-class D2D1_COLOR_F(Structure):
-    _fields_ = (
-        ('r', FLOAT),
-        ('g', FLOAT),
-        ('b', FLOAT),
-        ('a', FLOAT),
-    )
-
-
 class ID2D1Resource(com.pIUnknown):
     _methods_ = [
         ('GetFactory',
@@ -1288,11 +1346,6 @@ D2D1_DRAW_TEXT_OPTIONS_DISABLE_COLOR_BITMAP_SNAPPING = 0x00000008
 D2D1_DRAW_TEXT_OPTIONS_NONE = 0x00000000
 D2D1_DRAW_TEXT_OPTIONS_FORCE_DWORD = 0xffffffff
 
-DWRITE_MEASURING_MODE = UINT
-DWRITE_MEASURING_MODE_NATURAL = 0
-DWRITE_MEASURING_MODE_GDI_CLASSIC = 1
-DWRITE_MEASURING_MODE_GDI_NATURAL = 2
-
 
 class D2D1_PIXEL_FORMAT(Structure):
     _fields_ = (
@@ -1387,7 +1440,7 @@ class ID2D1RenderTarget(ID2D1Resource, com.pIUnknown):
         ('GetTransform',
          com.STDMETHOD()),
         ('SetAntialiasMode',
-         com.STDMETHOD()),
+         com.METHOD(c_void, D2D1_TEXT_ANTIALIAS_MODE)),
         ('GetAntialiasMode',
          com.STDMETHOD()),
         ('SetTextAntialiasMode',
@@ -1395,7 +1448,7 @@ class ID2D1RenderTarget(ID2D1Resource, com.pIUnknown):
         ('GetTextAntialiasMode',
          com.STDMETHOD()),
         ('SetTextRenderingParams',
-         com.STDMETHOD()),
+         com.STDMETHOD(IDWriteRenderingParams)),
         ('GetTextRenderingParams',
          com.STDMETHOD()),
         ('SetTags',
@@ -1407,7 +1460,7 @@ class ID2D1RenderTarget(ID2D1Resource, com.pIUnknown):
         ('PopLayer',
          com.STDMETHOD()),
         ('Flush',
-         com.STDMETHOD()),
+         com.STDMETHOD(c_void_p, c_void_p)),
         ('SaveDrawingState',
          com.STDMETHOD()),
         ('RestoreDrawingState',
@@ -1497,7 +1550,8 @@ if not wic_decoder:
 
 class DirectWriteGlyphRenderer(base.GlyphRenderer):
     antialias_mode = D2D1_TEXT_ANTIALIAS_MODE_DEFAULT
-    draw_options = D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT
+    draw_options = D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT if WINDOWS_8_1_OR_GREATER else D2D1_DRAW_TEXT_OPTIONS_NONE
+    measuring_mode = DWRITE_MEASURING_MODE_NATURAL
 
     def __init__(self, font):
         self._render_target = None
@@ -1533,10 +1587,13 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         width, height = int(math.ceil(layout_metrics.width)), int(math.ceil(layout_metrics.height))
 
         bitmap = IWICBitmap()
-        wic_decoder._factory.CreateBitmap(width, height,
-                                          GUID_WICPixelFormat32bppPBGRA,
-                                          WICBitmapCacheOnDemand,
-                                          byref(bitmap))
+        wic_decoder._factory.CreateBitmap(
+            width,
+            height,
+            GUID_WICPixelFormat32bppPBGRA,
+            WICBitmapCacheOnDemand,
+            byref(bitmap)
+        )
 
         rt = ID2D1RenderTarget()
         d2d_factory.CreateWicBitmapRenderTarget(bitmap, default_target_properties, byref(rt))
@@ -1549,14 +1606,11 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
 
         rt.CreateSolidColorBrush(white, None, byref(self._brush))
 
-        # This offsets the characters if needed.
-        point = D2D_POINT_2F(0, 0)
-
         rt.BeginDraw()
 
         rt.Clear(transparent)
 
-        rt.DrawTextLayout(point,
+        rt.DrawTextLayout(no_offset,
                           text_layout,
                           self._brush,
                           self.draw_options)
@@ -1569,8 +1623,7 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
 
         return image_data
 
-    def get_string_info(self, text):
-
+    def get_string_info(self, text, font_face):
         """Converts a string of text into a list of indices and advances used for shaping."""
         text_length = len(text.encode('utf-16-le')) // 2
 
@@ -1591,44 +1644,47 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         glyph_props = (DWRITE_SHAPING_GLYPH_PROPERTIES * max_glyph_size)()
         actual_count = UINT32()
 
-        self._analyzer.GetGlyphs(text_buffer,
-                                 length,
-                                 self.font.font_face,
-                                 False,  # sideways
-                                 False,  # righttoleft
-                                 self._text_analysis._script,  # scriptAnalysis
-                                 None,  # localName
-                                 None,  # numberSub
-                                 None,  # typo features
-                                 None,  # feature range length
-                                 0,  # feature range
-                                 max_glyph_size,  # max glyph size
-                                 clusters,  # cluster map
-                                 text_props,  # text props
-                                 indices,  # glyph indices
-                                 glyph_props,  # glyph pops
-                                 byref(actual_count)  # glyph count
-                                 )
+        self._analyzer.GetGlyphs(
+            text_buffer,
+            length,
+            font_face,
+            False,  # sideways
+            False,  # righttoleft
+            self._text_analysis._script,  # scriptAnalysis
+            None,  # localName
+            None,  # numberSub
+            None,  # typo features
+            None,  # feature range length
+            0,  # feature range
+            max_glyph_size,  # max glyph size
+            clusters,  # cluster map
+            text_props,  # text props
+            indices,  # glyph indices
+            glyph_props,  # glyph pops
+            byref(actual_count)  # glyph count
+        )
 
         advances = (FLOAT * length)()
         offsets = (DWRITE_GLYPH_OFFSET * length)()
-        self._analyzer.GetGlyphPlacements(text_buffer,
-                                          clusters,
-                                          text_props,
-                                          text_length,
-                                          indices,
-                                          glyph_props,
-                                          actual_count,
-                                          self.font.font_face,
-                                          self.font._font_metrics.designUnitsPerEm,
-                                          False, False,
-                                          self._text_analysis._script,
-                                          self.font.locale,
-                                          None,
-                                          None,
-                                          0,
-                                          advances,
-                                          offsets)
+        self._analyzer.GetGlyphPlacements(
+            text_buffer,
+            clusters,
+            text_props,
+            text_length,
+            indices,
+            glyph_props,
+            actual_count,
+            font_face,
+            self.font._font_metrics.designUnitsPerEm,
+            False, False,
+            self._text_analysis._script,
+            self.font.locale,
+            None,
+            None,
+            0,
+            advances,
+            offsets
+        )
 
         return text_buffer, actual_count.value, indices, advances, offsets, clusters
 
@@ -1640,7 +1696,6 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         font_face.GetDesignGlyphMetrics(indices, count, glyph_metrics, False)
 
         metrics_out = []
-        i = 0
         for metric in glyph_metrics:
             glyph_width = (metric.advanceWidth - metric.leftSideBearing - metric.rightSideBearing)
 
@@ -1657,7 +1712,6 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
             advance_width = metric.advanceWidth
 
             metrics_out.append((glyph_width, glyph_height, lsb, advance_width, bsb))
-            i += 1
 
         return metrics_out
 
@@ -1674,28 +1728,59 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         )
         return run
 
+    def is_color_run(self, run):
+        """Will return True if the run contains a colored glyph."""
+        enumerator = IDWriteColorGlyphRunEnumerator1()
+        try:
+            color = self.font._write_factory.TranslateColorGlyphRun4(no_offset,
+                                                                     run,
+                                                                     None,
+                                                                     DWRITE_GLYPH_IMAGE_FORMATS_ALL,
+                                                                     self.measuring_mode,
+                                                                     None,
+                                                                     0,
+                                                                     enumerator)
+
+            return True
+        except OSError:
+            # HRESULT returns -2003283956 (DWRITE_E_NOCOLOR) if no color run is detected.
+            pass
+
+        return False
+
     def render_single_glyph(self, font_face, indice, advance, offset, metrics):
         """Renders a single glyph using D2D DrawGlyphRun"""
-        glyph_width, glyph_height, lsb, font_advance, bsb = metrics  # We use a shaped advance instead of the fonts.
+        glyph_width, glyph_height, glyph_lsb, glyph_advance, glyph_bsb = metrics  # We use a shaped advance instead of the fonts.
 
         # Slicing an array turns it into a python object. Maybe a better way to keep it a ctypes value?
         new_indice = (UINT16 * 1)(indice)
         new_advance = (FLOAT * 1)(advance)
 
-        run = self._get_single_glyph_run(font_face,
-                                         self.font._real_size,
-                                         new_indice,  # indice,
-                                         new_advance,  # advance,
-                                         pointer(offset),  # offset,
-                                         False,
-                                         False)
+        run = self._get_single_glyph_run(
+            font_face,
+            self.font._real_size,
+            new_indice,  # indice,
+            new_advance,  # advance,
+            pointer(offset),  # offset,
+            False,
+            0
+        )
 
-        render_width = int(math.ceil((glyph_width) * self.font.font_scale_ratio))
-        render_offset_x = int(math.floor(abs(lsb * self.font.font_scale_ratio)))
-        if lsb < 0:
-            # Negative LSB: we shift the layout rect to the right
-            # Otherwise we will cut the left part of the glyph
-            render_offset_x = -(render_offset_x)
+        # If it's colored, return to render it using layout.
+        if self.draw_options & D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT and self.is_color_run(run):
+            return None
+
+        # Use the glyph's advance as a width as bitmap width.
+        # Some characters such as diacritics (̃) may have 0 advance width. In that case, just use glyph_width
+        if glyph_advance:
+            render_width = int(math.ceil(glyph_advance * self.font.font_scale_ratio))
+        else:
+            render_width = int(math.ceil(glyph_width * self.font.font_scale_ratio))
+
+        render_offset_x = 0
+        if glyph_lsb < 0:
+            # Negative LSB: we shift the offset, otherwise the glyph will be cut off.
+            render_offset_x = glyph_lsb * self.font.font_scale_ratio
 
         # Create new bitmap.
         # TODO: We can probably adjust bitmap/baseline to reduce the whitespace and save a lot of texture space.
@@ -1704,7 +1789,6 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
                             int(math.ceil(self.font.max_glyph_height)))
 
         # Glyphs are drawn at the baseline, and with LSB, so we need to offset it based on top left position.
-        # Offsets are actually based on pixels somehow???
         baseline_offset = D2D_POINT_2F(-render_offset_x - offset.advanceOffset,
                                        self.font.ascent + offset.ascenderOffset)
 
@@ -1715,17 +1799,17 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         self._render_target.DrawGlyphRun(baseline_offset,
                                          run,
                                          self._brush,
-                                         DWRITE_MEASURING_MODE_NATURAL)
+                                         self.measuring_mode)
 
         self._render_target.EndDraw(None, None)
         image = wic_decoder.get_image(self._bitmap)
 
         glyph = self.font.create_glyph(image)
 
-        glyph.set_bearings(self.font.descent, render_offset_x,
-                           advance * self.font.font_scale_ratio,
-                           offset.advanceOffset * self.font.font_scale_ratio,
-                           offset.ascenderOffset * self.font.font_scale_ratio)
+        glyph.set_bearings(-self.font.descent, render_offset_x,
+                           advance,
+                           offset.advanceOffset,
+                           offset.ascenderOffset)
 
         return glyph
 
@@ -1738,8 +1822,13 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         layout_metrics = DWRITE_TEXT_METRICS()
         text_layout.GetMetrics(byref(layout_metrics))
 
-        self._create_bitmap(int(math.ceil(layout_metrics.width)),
-                            int(math.ceil(layout_metrics.height)))
+        width = int(math.ceil(layout_metrics.width))
+        height = int(math.ceil(layout_metrics.height))
+
+        if width == 0 or height == 0:
+            return None
+
+        self._create_bitmap(width, height)
 
         # This offsets the characters if needed.
         point = D2D_POINT_2F(0, 0)
@@ -1758,7 +1847,17 @@ class DirectWriteGlyphRenderer(base.GlyphRenderer):
         image = wic_decoder.get_image(self._bitmap)
 
         glyph = self.font.create_glyph(image)
-        glyph.set_bearings(self.font.descent, 0, int(math.ceil(layout_metrics.width)))
+        glyph.set_bearings(-self.font.descent, 0, int(math.ceil(layout_metrics.width)))
+        return glyph
+
+    def create_zero_glyph(self):
+        """Zero glyph is a 1x1 image that has a -1 advance. This is to fill in for ligature substitutions since
+        font system requires 1 glyph per character in a string."""
+        self._create_bitmap(1, 1)
+        image = wic_decoder.get_image(self._bitmap)
+
+        glyph = self.font.create_glyph(image)
+        glyph.set_bearings(-self.font.descent, 0, -1)
         return glyph
 
     def _create_bitmap(self, width, height):
@@ -1806,6 +1905,8 @@ class Win32DirectWriteFont(base.Font):
     _default_name = 'Segoe UI'  # Default font for Win7/10.
 
     _glyph_renderer = None
+    _empty_glyph = None
+    _zero_glyph = None
 
     glyph_renderer_class = DirectWriteGlyphRenderer
     texture_internalformat = pyglet.gl.GL_RGBA
@@ -1821,7 +1922,7 @@ class Win32DirectWriteFont(base.Font):
         self._font_index, self._collection = self.get_collection(name)
         assert self._collection is not None, "Font: {} not found in loaded or system font collection.".format(name)
 
-        self.name = name
+        self._name = name
         self.bold = bold
         self.size = size
         self.italic = italic
@@ -1867,24 +1968,28 @@ class Win32DirectWriteFont(base.Font):
         # Create the text format this font will use permanently.
         # Could technically be recreated, but will keep to be inline with other font objects.
         self._text_format = IDWriteTextFormat()
-        self._write_factory.CreateTextFormat(self.name,
-                                             self._collection,
-                                             self._weight,
-                                             self._style,
-                                             self._stretch,
-                                             self._real_size,
-                                             create_unicode_buffer(self.locale),
-                                             byref(self._text_format))
+        self._write_factory.CreateTextFormat(
+            self._name,
+            self._collection,
+            self._weight,
+            self._style,
+            self._stretch,
+            self._real_size,
+            create_unicode_buffer(self.locale),
+            byref(self._text_format)
+        )
 
-        # All this work just to get a font face and it's metrics!
+        # All this work just to get a font face and its metrics!
         font_family = IDWriteFontFamily1()
         self._collection.GetFontFamily(self._font_index, byref(font_family))
 
         write_font = IDWriteFont()
-        font_family.GetFirstMatchingFont(self._weight,
-                                         self._stretch,
-                                         self._style,
-                                         byref(write_font))
+        font_family.GetFirstMatchingFont(
+            self._weight,
+            self._stretch,
+            self._style,
+            byref(write_font)
+        )
 
         font_face = IDWriteFontFace()
         write_font.CreateFontFace(byref(font_face))
@@ -1897,12 +2002,23 @@ class Win32DirectWriteFont(base.Font):
 
         self.font_scale_ratio = (self._real_size / self._font_metrics.designUnitsPerEm)
 
-        self.ascent = self._font_metrics.ascent * self.font_scale_ratio
-        self.descent = self._font_metrics.descent * self.font_scale_ratio
-
+        self.ascent = math.ceil(self._font_metrics.ascent * self.font_scale_ratio)
+        self.descent = -round(self._font_metrics.descent * self.font_scale_ratio)
         self.max_glyph_height = (self._font_metrics.ascent + self._font_metrics.descent) * self.font_scale_ratio
+
         self.line_gap = self._font_metrics.lineGap * self.font_scale_ratio
 
+        self._fallback = None
+        if WINDOWS_8_1_OR_GREATER:
+            self._fallback = IDWriteFontFallback()
+            self._write_factory.GetSystemFontFallback(byref(self._fallback))
+        else:
+            assert _debug_font("Windows 8.1+ is required for font fallback. Colored glyphs cannot be omitted.")
+
+    @property
+    def name(self):
+        return self._name
+
     def render_to_image(self, text, width=10000, height=80):
         """This process takes Pyglet out of the equation and uses only DirectWrite to shape and render text.
         This may allow more accurate fonts (bidi, rtl, etc) in very special circumstances at the cost of
@@ -1924,51 +2040,159 @@ class Win32DirectWriteFont(base.Font):
         """This takes the existing glyph texture and puts it into a new Glyph with a new advance.
         Texture memory is shared between both glyphs."""
         new_glyph = base.Glyph(glyph.x, glyph.y, glyph.z, glyph.width, glyph.height, glyph.owner)
-        new_glyph.set_bearings(glyph.baseline,
-                               glyph.lsb,
-                               advance * self.font_scale_ratio,
-                               offset.advanceOffset * self.font_scale_ratio,
-                               offset.ascenderOffset * self.font_scale_ratio)
+        new_glyph.set_bearings(
+            glyph.baseline,
+            glyph.lsb,
+            advance,
+            offset.advanceOffset,
+            offset.ascenderOffset
+        )
         return new_glyph
 
+    def _render_layout_glyph(self, text_buffer, i, clusters, check_color=True):
+        # Some glyphs can be more than 1 char. We use the clusters to determine how many of an index exist.
+        text_length = clusters.count(i)
+
+        # Amount of glyphs don't always match 1:1 with text as some can be substituted or omitted. Get
+        # actual text buffer index.
+        text_index = clusters.index(i)
+
+        # Get actual text based on the index and length.
+        actual_text = text_buffer[text_index:text_index + text_length]
+
+        # Since we can't store as indice 0 without overriding, we have to store as text
+        if actual_text not in self.glyphs:
+            glyph = self._glyph_renderer.render_using_layout(text_buffer[text_index:text_index + text_length])
+            if glyph:
+                if check_color and self._glyph_renderer.draw_options & D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT:
+                    fb_ff = self._get_fallback_font_face(text_index, text_length)
+                    if fb_ff:
+                        glyph.colored = self.is_fallback_str_colored(fb_ff, actual_text)
+            else:
+                glyph = self._empty_glyph
+
+            self.glyphs[actual_text] = glyph
+
+        return self.glyphs[actual_text]
+
+    def is_fallback_str_colored(self, font_face, text):
+        indice = UINT16()
+        code_points = (UINT32 * len(text))(*[ord(c) for c in text])
+
+        font_face.GetGlyphIndices(code_points, len(text), byref(indice))
+
+        new_indice = (UINT16 * 1)(indice)
+        new_advance = (FLOAT * 1)(100)  # dummy
+        offset = (DWRITE_GLYPH_OFFSET * 1)()
+
+        run = self._glyph_renderer._get_single_glyph_run(
+            font_face,
+            self._real_size,
+            new_indice,  # indice,
+            new_advance,  # advance,
+            offset,  # offset,
+            False,
+            False
+        )
+
+        return self._glyph_renderer.is_color_run(run)
+
+    def _get_fallback_font_face(self, text_index, text_length):
+        if WINDOWS_8_1_OR_GREATER:
+            out_length = UINT32()
+            fb_font = IDWriteFont()
+            scale = FLOAT()
+
+            self._fallback.MapCharacters(
+                self._glyph_renderer._text_analysis,
+                text_index,
+                text_length,
+                None,
+                None,
+                self._weight,
+                self._style,
+                self._stretch,
+                byref(out_length),
+                byref(fb_font),
+                byref(scale)
+            )
+
+            if fb_font:
+                fb_font_face = IDWriteFontFace()
+                fb_font.CreateFontFace(byref(fb_font_face))
+
+                return fb_font_face
+
+        return None
+
+    def get_glyphs_no_shape(self, text):
+        """This differs in that it does not attempt to shape the text at all. May be useful in cases where your font
+        has no special shaping requirements, spacing is the same, or some other reason where faster performance is
+        wanted and you can get away with this."""
+        if not self._glyph_renderer:
+            self._glyph_renderer = self.glyph_renderer_class(self)
+            self._empty_glyph = self._glyph_renderer.render_using_layout(" ")
+
+        glyphs = []
+        for c in text:
+            if c == '\t':
+                c = ' '
+
+            if c not in self.glyphs:
+                self.glyphs[c] = self._glyph_renderer.render_using_layout(c)
+                if not self.glyphs[c]:
+                    self.glyphs[c] = self._empty_glyph
+
+            glyphs.append(self.glyphs[c])
+
+        return glyphs
+
     def get_glyphs(self, text):
         if not self._glyph_renderer:
             self._glyph_renderer = self.glyph_renderer_class(self)
+            self._empty_glyph = self._glyph_renderer.render_using_layout(" ")
+            self._zero_glyph = self._glyph_renderer.create_zero_glyph()
 
-        text_buffer, actual_count, indices, advances, offsets, clusters = self._glyph_renderer.get_string_info(text)
+        text_buffer, actual_count, indices, advances, offsets, clusters = self._glyph_renderer.get_string_info(text,
+                                                                                                               self.font_face)
 
         metrics = self._glyph_renderer.get_glyph_metrics(self.font_face, indices, actual_count)
 
+        formatted_clusters = list(clusters)
+
+        # Convert to real sizes.
+        for i in range(actual_count):
+            advances[i] *= self.font_scale_ratio
+
+        for i in range(actual_count):
+            offsets[i].advanceOffset *= self.font_scale_ratio
+            offsets[i].ascenderOffset *= self.font_scale_ratio
+
         glyphs = []
+
+        # Pyglet expects 1 glyph for every string. However, ligatures can combine 1 or more glyphs, leading
+        # to issues with multilines producing wrong output.
+        substitutions = {}
+        for idx in clusters:
+            ct = formatted_clusters.count(idx)
+            if ct > 1:
+                substitutions[idx] = ct-1
+
         for i in range(actual_count):
             indice = indices[i]
+
             if indice == 0:
                 # If an indice is 0, it will return no glyph. In this case we attempt to render leveraging
                 # the built in text layout from MS. Which depending on version can use fallback fonts and other tricks
                 # to possibly get something of use.
-                formatted_clusters = clusters[:]
-
-                # Some glyphs can be more than 1 char. We use the clusters to determine how many of an index exist.
-                text_length = formatted_clusters.count(i)
-
-                # Amount of glyphs don't always match 1:1 with text as some can be substituted or omitted. Get
-                # actual text buffer index.
-                text_index = formatted_clusters.index(i)
-
-                # Get actual text based on the index and length.
-                actual_text = text_buffer[text_index:text_index + text_length]
-
-                # Since we can't store as indice 0 without overriding, we have to store as text
-                if actual_text not in self.glyphs:
-                    glyph = self._glyph_renderer.render_using_layout(text_buffer[text_index:text_index + text_length])
-                    self.glyphs[actual_text] = glyph
-
-                glyphs.append(self.glyphs[actual_text])
+                glyph = self._render_layout_glyph(text_buffer, i, formatted_clusters)
+                glyphs.append(glyph)
             else:
+                advance_key = (indice, advances[i], offsets[i].advanceOffset, offsets[i].ascenderOffset)
+
                 # Glyphs can vary depending on shaping. We will cache it by indice, advance, and offset.
                 # Possible to just cache without offset and set them each time. This may be faster?
                 if indice in self.glyphs:
-                    advance_key = (indice, advances[i], offsets[i].advanceOffset, offsets[i].ascenderOffset)
                     if advance_key in self._advance_cache:
                         glyph = self._advance_cache[advance_key]
                     else:
@@ -1977,11 +2201,19 @@ class Win32DirectWriteFont(base.Font):
                 else:
                     glyph = self._glyph_renderer.render_single_glyph(self.font_face, indice, advances[i], offsets[i],
                                                                      metrics[i])
+                    if glyph is None:  # Will only return None if a color glyph is found. Use DW to render it directly.
+                        glyph = self._render_layout_glyph(text_buffer, i, formatted_clusters, check_color=False)
+                        glyph.colored = True
+
                     self.glyphs[indice] = glyph
-                    self._advance_cache[(indice, advances[i], offsets[i].advanceOffset, offsets[i].ascenderOffset)] = glyph
+                    self._advance_cache[advance_key] = glyph
 
                 glyphs.append(glyph)
 
+            if i in substitutions:
+                for _ in range(substitutions[i]):
+                    glyphs.append(self._zero_glyph)
+
         return glyphs
 
     def create_text_layout(self, text):
@@ -2092,7 +2324,7 @@ class Win32DirectWriteFont(base.Font):
 
     @classmethod
     def get_collection(cls, font_name):
-        """Returns which collection this font belongs to (system or custom collection), as well as it's index in the
+        """Returns which collection this font belongs to (system or custom collection), as well as its index in the
         collection."""
         if not cls._write_factory:
             cls._initialize_direct_write()
@@ -2184,3 +2416,8 @@ WICBitmapCacheOnLoad = 0x2
 
 transparent = D2D1_COLOR_F(0.0, 0.0, 0.0, 0.0)
 white = D2D1_COLOR_F(1.0, 1.0, 1.0, 1.0)
+no_offset = D2D_POINT_2F(0, 0)
+
+# If we are not shaping, monkeypatch to no shape function.
+if pyglet.options["win32_disable_shaping"]:
+    Win32DirectWriteFont.get_glyphs = Win32DirectWriteFont.get_glyphs_no_shape
