@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2021 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -120,7 +120,18 @@ class FreeTypeGlyphRenderer(base.GlyphRenderer):
                               'A',
                               self._data,
                               abs(self._pitch))
-        glyph = self.font.create_glyph(img)
+
+        # HACK: Get text working in GLES until image data can be converted properly
+        #       GLES don't support coversion during pixel transfer so we have to
+        #       force specify the glyph format to be GL_ALPHA. This format is not
+        #       supported in 3.3+ core, but are present in ES because of pixel transfer
+        #       limitations.
+        if pyglet.gl.current_context.get_info().get_opengl_api() == "gles":
+            GL_ALPHA = 0x1906
+            glyph = self.font.create_glyph(img, fmt=GL_ALPHA)
+        else:
+            glyph = self.font.create_glyph(img)
+
         glyph.set_bearings(self._baseline, self._lsb, self._advance_x)
         if self._pitch > 0:
             t = list(glyph.tex_coords)
@@ -250,6 +261,10 @@ class FreeTypeFace:
                 raise base.FontException('No filename for "%s"' % match.name)
             return cls.from_file(match.file)
 
+    @property
+    def name(self):
+        return self._name
+
     @property
     def family_name(self):
         return asstr(self.ft_face.contents.family_name)
@@ -324,7 +339,7 @@ class FreeTypeFace:
                                    descent=-ascent // 4)  # arbitrary.
 
     def _get_best_name(self):
-        self._name = self.family_name
+        self._name = asstr(self.ft_face.contents.family_name)
         self._get_font_family_from_ttf
 
     def _get_font_family_from_ttf(self):
