@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2020 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -36,10 +36,8 @@
 """Low-level graphics rendering.
 
 This module provides an efficient low-level abstraction over OpenGL.  It gives
-very good performance for rendering OpenGL primitives; far better than the
-typical immediate-mode usage and, on modern graphics cards, better than using
-display lists in many cases.  The module is used internally by other areas of
-pyglet.
+very good performance for rendering OpenGL primitives. The module is used
+internally by other areas of pyglet.
 
 See the :ref:`guide_graphics` for details on how to use this graphics API.
 
@@ -47,10 +45,12 @@ Batches and groups
 ==================
 
 Without even needing to understand the details on how to draw primitives with
-the graphics API, developers can make use of :py:class:`~pyglet.graphics.Batch` and :py:class:`~pyglet.graphics.Group`
-objects to improve performance of sprite and text rendering.
+the graphics API, developers can make use of :py:class:`~pyglet.graphics.Batch`
+and :py:class:`~pyglet.graphics.Group` objects to improve performance of sprite
+and text rendering.
 
-The :py:class:`~pyglet.sprite.Sprite`, :py:func:`~pyglet.text.Label` and :py:func:`~pyglet.text.layout.TextLayout` classes all accept a ``batch`` and
+The :py:class:`~pyglet.sprite.Sprite`, :py:func:`~pyglet.text.Label` and
+:py:func:`~pyglet.text.layout.TextLayout` classes all accept a ``batch`` and
 ``group`` parameter in their constructors.  A batch manages a set of objects
 that will be drawn all at once, and a group describes the manner in which an
 object is drawn.
@@ -69,16 +69,16 @@ Drawing a complete batch is much faster than drawing the items in the batch
 individually, especially when those items belong to a common group.  
 
 Groups describe the OpenGL state required for an item.  This is for the most
-part managed by the sprite and text classes, however you can also use groups
-to ensure items are drawn in a particular order.  For example,  the following
-example adds a background sprite which is guaranteed to be drawn before the
-car and the boat::
+part managed by the sprite and text classes, however you can also use custom
+groups to ensure items are drawn in a particular order. For example, the
+following example adds a background sprite which is guaranteed to be drawn
+before the car and the boat::
 
     batch = pyglet.graphics.Batch()
-    background = pyglet.graphics.OrderedGroup(0)
-    foreground = pyglet.graphics.OrderedGroup(1)
+    background = pyglet.sprite.SpriteGroup(0)
+    foreground = pyglet.sprite.SpriteGroup(1)
 
-    background = pyglet.sprite.Sprite(background_image, 
+    background = pyglet.sprite.Sprite(background_image,
                                       batch=batch, group=background)
     car = pyglet.sprite.Sprite(car_image, batch=batch, group=foreground)
     boat = pyglet.sprite.Sprite(boat_image, batch=batch, group=foreground)
@@ -91,46 +91,13 @@ possible.  If the drawing of sprites or text objects need to be interleaved
 with other drawing that does not use the graphics API, multiple batches will
 be required.
 
-Data item parameters
-====================
-
-Many of the functions and methods in this module accept any number of ``data``
-parameters as their final parameters.  In the documentation these are notated
-as ``*data`` in the formal parameter list.
-
-A data parameter describes a vertex attribute format and an optional sequence
-to initialise that attribute.  Examples of common attribute formats are:
-
-``"v3f"``
-    Vertex position, specified as three floats.
-``"c4B"``
-    Vertex color, specified as four unsigned bytes.
-``"t2f"``
-    Texture coordinate, specified as two floats.
-
-See `pyglet.graphics.vertexattribute` for the complete syntax of the vertex
-format string.
-
-When no initial data is to be given, the data item is just the format string.
-For example, the following creates a 2 element vertex list with position and
-color attributes::
-
-    vertex_list = pyglet.graphics.vertex_list(2, 'v2f', 'c4B')
-
-When initial data is required, wrap the format string and the initial data in
-a tuple, for example::
-
-    vertex_list = pyglet.graphics.vertex_list(2, 
-                                              ('v2f', (0.0, 1.0, 1.0, 0.0)),
-                                              ('c4B', (255, 255, 255, 255) * 2))
-
 Drawing modes
 =============
 
 Methods in this module that accept a ``mode`` parameter will accept any value
 in the OpenGL drawing mode enumeration: ``GL_POINTS``, ``GL_LINE_STRIP``,
 ``GL_LINE_LOOP``, ``GL_LINES``, ``GL_TRIANGLE_STRIP``, ``GL_TRIANGLE_FAN``,
-``GL_TRIANGLES``, ``GL_QUAD_STRIP``, ``GL_QUADS``, and ``GL_POLYGON``.
+``GL_TRIANGLES``, and ``GL_POLYGON``.
 
 :: 
 
@@ -140,8 +107,8 @@ However, because of the way the graphics API renders multiple primitives with
 shared state, ``GL_POLYGON``, ``GL_LINE_LOOP`` and ``GL_TRIANGLE_FAN`` cannot
 be used --- the results are undefined.
 
-When using ``GL_LINE_STRIP``, ``GL_TRIANGLE_STRIP`` or ``GL_QUAD_STRIP`` care
-must be taken to insert degenerate vertices at the beginning and end of each
+When using ``GL_LINE_STRIP`` or ``GL_TRIANGLE_STRIP``, care must be taken to
+insert degenerate vertices at the beginning and end of each
 vertex list.  For example, given the vertex list::
 
     A, B, C, D
@@ -159,16 +126,18 @@ video drivers, and requires indexed vertex lists.
 """
 
 import ctypes
+import weakref
 
 import pyglet
 from pyglet.gl import *
-from pyglet import gl
-from pyglet.graphics import vertexbuffer, vertexattribute, vertexdomain
+from pyglet.graphics import shader, vertexdomain
+from pyglet.graphics.vertexarray import VertexArray
+from pyglet.graphics.vertexbuffer import BufferObject
 
 _debug_graphics_batch = pyglet.options['debug_graphics_batch']
 
 
-def draw(size, mode, *data):
+def draw(size, mode, **data):
     """Draw a primitive immediately.
 
     :Parameters:
@@ -177,31 +146,47 @@ def draw(size, mode, *data):
         `mode` : gl primitive type 
             OpenGL drawing mode, e.g. ``GL_TRIANGLES``, 
             avoiding quotes.
-        `data` : data items
-            Attribute formats and data.  See the module summary for 
-            details.
+        `**data` : keyword arguments for passing vertex attribute data.
+            The keyword should be the vertex attribute name, and the
+            argument should be a tuple of (format, data). For example:
+            `position=('f', array)`
 
     """
-    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
+    # Create and bind a throwaway VAO
+    vao_id = GLuint()
+    glGenVertexArrays(1, vao_id)
+    glBindVertexArray(vao_id)
+    # Activate shader program:
+    program = get_default_shader()
+    program.use()
 
     buffers = []
-    for fmt, array in data:
-        attribute = vertexattribute.create_attribute(fmt)
+    for name, (fmt, array) in data.items():
+        location = program.attributes[name]['location']
+        count = program.attributes[name]['count']
+        gl_type = vertexdomain._gl_types[fmt[0]]
+        normalize = 'n' in fmt
+        attribute = shader.Attribute(name, location, count, gl_type, normalize)
         assert size == len(array) // attribute.count, 'Data for %s is incorrect length' % fmt
-        buffer = vertexbuffer.create_mappable_buffer(size * attribute.stride, vbo=False)
 
+        buffer = BufferObject(size * attribute.stride)
         attribute.set_region(buffer, 0, size, array)
         attribute.enable()
         attribute.set_pointer(buffer.ptr)
-        buffers.append(buffer)
+
+        buffers.append(buffer)      # Don't garbage collect it.
 
     glDrawArrays(mode, 0, size)
-    glFlush()
 
-    glPopClientAttrib()
+    # Deactivate shader program:
+    program.stop()
+    # Discard everything after drawing:
+    del buffers
+    glBindVertexArray(0)
+    glDeleteVertexArrays(1, vao_id)
 
 
-def draw_indexed(size, mode, indices, *data):
+def draw_indexed(size, mode, indices, **data):
     """Draw a primitive with indexed vertices immediately.
 
     :Parameters:
@@ -211,18 +196,30 @@ def draw_indexed(size, mode, indices, *data):
             OpenGL drawing mode, e.g. ``GL_TRIANGLES``
         `indices` : sequence of int
             Sequence of integers giving indices into the vertex list.
-        `data` : data items
-            Attribute formats and data.  See the module summary for details.
+        `**data` : keyword arguments for passing vertex attribute data.
+            The keyword should be the vertex attribute name, and the
+            argument should be a tuple of (format, data). For example:
+            `position=('f', array)`
 
     """
-    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
+    # Create and bind a throwaway VAO
+    vao_id = GLuint()
+    glGenVertexArrays(1, vao_id)
+    glBindVertexArray(vao_id)
+    # Activate shader program:
+    program = get_default_shader()
+    program.use()
 
     buffers = []
-    for fmt, array in data:
-        attribute = vertexattribute.create_attribute(fmt)
+    for name, (fmt, array) in data.items():
+        location = program.attributes[name]['location']
+        count = program.attributes[name]['count']
+        gl_type = vertexdomain._gl_types[fmt[0]]
+        normalize = 'n' in fmt
+        attribute = shader.Attribute(name, location, count, gl_type, normalize)
         assert size == len(array) // attribute.count, 'Data for %s is incorrect length' % fmt
-        buffer = vertexbuffer.create_mappable_buffer(size * attribute.stride, vbo=False)
 
+        buffer = BufferObject(size * attribute.stride)
         attribute.set_region(buffer, 0, size, array)
         attribute.enable()
         attribute.set_pointer(buffer.ptr)
@@ -238,82 +235,98 @@ def draw_indexed(size, mode, indices, *data):
         index_type = GL_UNSIGNED_INT
         index_c_type = ctypes.c_uint
 
+    # With GL 3.3 vertex arrays indices needs to be in a buffer
+    # bound to the ELEMENT_ARRAY slot
     index_array = (index_c_type * len(indices))(*indices)
-    glDrawElements(mode, len(indices), index_type, index_array)
+    index_buffer = BufferObject(ctypes.sizeof(index_array))
+    index_buffer.set_data(index_array)
+    index_buffer.bind_to_index_buffer()
+
+    glDrawElements(mode, len(indices), index_type, 0)
     glFlush()
 
-    glPopClientAttrib()
+    # Deactivate shader program:
+    program.stop()
+    # Discard everything after drawing:
+    del buffers
+    del index_buffer
+    glBindVertexArray(0)
+    glDeleteVertexArrays(1, vao_id)
 
 
-def _parse_data(data):
-    """Given a list of data items, returns (formats, initial_arrays)."""
-    assert data, 'No attribute formats given'
+# Default Shader source:
 
-    # Return tuple (formats, initial_arrays).
-    formats = []
-    initial_arrays = []
-    for i, fmt in enumerate(data):
-        if isinstance(fmt, tuple):
-            fmt, array = fmt
-            initial_arrays.append((i, array))
-        formats.append(fmt)
-    formats = tuple(formats)
-    return formats, initial_arrays
+_vertex_source = """#version 330 core
+    in vec3 position;
+    in vec4 colors;
+    in vec3 tex_coords;
+    out vec4 vertex_colors;
+    out vec3 texture_coords;
+
+    uniform WindowBlock
+    {
+        mat4 projection;
+        mat4 view;
+    } window;  
+
+    void main()
+    {
+        gl_Position = window.projection * window.view * vec4(position, 1.0);
+
+        vertex_colors = colors;
+        texture_coords = tex_coords;
+    }
+"""
+
+_fragment_source = """#version 330 core
+    in vec4 vertex_colors;
+    in vec3 texture_coords;
+    out vec4 final_colors;
+
+    uniform sampler2D our_texture;
+
+    void main()
+    {
+        final_colors = texture(our_texture, texture_coords.xy) + vertex_colors;
+    }
+"""
 
 
-def _get_default_batch():
-    shared_object_space = gl.current_context.object_space
+def get_default_batch():
     try:
-        return shared_object_space.pyglet_graphics_default_batch
+        return pyglet.gl.current_context.pyglet_graphics_default_batch
     except AttributeError:
-        shared_object_space.pyglet_graphics_default_batch = Batch()
-        return shared_object_space.pyglet_graphics_default_batch
+        pyglet.gl.current_context.pyglet_graphics_default_batch = Batch()
+        return pyglet.gl.current_context.pyglet_graphics_default_batch
 
 
-def vertex_list(count, *data):
-    """Create a :py:class:`~pyglet.graphics.vertexdomain.VertexList` not associated with a batch, group or mode.
-
-    :Parameters:
-        `count` : int
-            The number of vertices in the list.
-        `data` : data items
-            Attribute formats and initial data for the vertex list.  See the
-            module summary for details.
-
-    :rtype: :py:class:`~pyglet.graphics.vertexdomain.VertexList`
-    """
-    # Note that mode=0 because the default batch is never drawn: vertex lists
-    # returned from this function are drawn directly by the app.
-    return _get_default_batch().add(count, 0, None, *data)
+def get_default_group():
+    try:
+        return pyglet.gl.current_context.pyglet_graphics_default_group
+    except AttributeError:
+        pyglet.gl.current_context.pyglet_graphics_default_group = ShaderGroup(get_default_shader())
+        return pyglet.gl.current_context.pyglet_graphics_default_group
 
 
-def vertex_list_indexed(count, indices, *data):
-    """Create an `IndexedVertexList` not associated with a batch, group or mode.
-
-    :Parameters:
-        `count` : int
-            The number of vertices in the list.
-        `indices` : sequence
-            Sequence of integers giving indices into the vertex list.
-        `data` : data items
-            Attribute formats and initial data for the vertex list.  See the
-            module summary for details.
-
-    :rtype: `IndexedVertexList`
-    """
-    # Note that mode=0 because the default batch is never drawn: vertex lists
-    # returned from this function are drawn directly by the app.
-    return _get_default_batch().add_indexed(count, 0, None, indices, *data)
+def get_default_shader():
+    try:
+        return pyglet.gl.current_context.pyglet_graphics_default_shader
+    except AttributeError:
+        _default_vert_shader = pyglet.graphics.shader.Shader(_vertex_source, 'vertex')
+        _default_frag_shader = pyglet.graphics.shader.Shader(_fragment_source, 'fragment')
+        default_shader_program = pyglet.graphics.shader.ShaderProgram(_default_vert_shader, _default_frag_shader)
+        pyglet.gl.current_context.pyglet_graphics_default_shader = default_shader_program
+        return pyglet.gl.current_context.pyglet_graphics_default_shader
 
 
 class Batch:
     """Manage a collection of vertex lists for batched rendering.
 
-    Vertex lists are added to a :py:class:`~pyglet.graphics.Batch` using the `add` and `add_indexed`
-    methods.  An optional group can be specified along with the vertex list,
-    which gives the OpenGL state required for its rendering.  Vertex lists
-    with shared mode and group are allocated into adjacent areas of memory and
-    sent to the graphics card in a single operation.
+    Vertex lists are added to a :py:class:`~pyglet.graphics.Batch` using the
+    `add` and `add_indexed` methods. An optional group can be specified along
+    with the vertex list, which gives the OpenGL state required for its rendering.
+    Vertex lists with shared mode and group are allocated into adjacent areas of
+    memory and sent to the graphics card in a single operation.
 
     Call `VertexList.delete` to remove a vertex list from the batch.
     """
@@ -333,6 +346,8 @@ class Batch:
         self._draw_list = []
         self._draw_list_dirty = False
 
+        self._context = pyglet.gl.current_context
+
     def invalidate(self):
         """Force the batch to update the draw list.
 
@@ -343,66 +358,6 @@ class Batch:
         """
         self._draw_list_dirty = True
 
-    def add(self, count, mode, group, *data):
-        """Add a vertex list to the batch.
-
-        :Parameters:
-            `count` : int
-                The number of vertices in the list.
-            `mode` : int
-                OpenGL drawing mode enumeration; for example, one of
-                ``GL_POINTS``, ``GL_LINES``, ``GL_TRIANGLES``, etc.
-                See the module summary for additional information.
-            `group` : `~pyglet.graphics.Group`
-                Group of the vertex list, or ``None`` if no group is required.
-            `data` : data items
-                Attribute formats and initial data for the vertex list.  See
-                the module summary for details.
-
-        :rtype: :py:class:`~pyglet.graphics.vertexdomain.VertexList`
-        """
-        formats, initial_arrays = _parse_data(data)
-        domain = self._get_domain(False, mode, group, formats)
-
-        # Create vertex list and initialize
-        vlist = domain.create(count)
-        for i, array in initial_arrays:
-            vlist._set_attribute_data(i, array)
-
-        return vlist
-
-    def add_indexed(self, count, mode, group, indices, *data):
-        """Add an indexed vertex list to the batch.
-
-        :Parameters:
-            `count` : int
-                The number of vertices in the list.
-            `mode` : int
-                OpenGL drawing mode enumeration; for example, one of
-                ``GL_POINTS``, ``GL_LINES``, ``GL_TRIANGLES``, etc.
-                See the module summary for additional information.
-            `group` : `~pyglet.graphics.Group`
-                Group of the vertex list, or ``None`` if no group is required.
-            `indices` : sequence
-                Sequence of integers giving indices into the vertex list.
-            `data` : data items
-                Attribute formats and initial data for the vertex list.  See
-                the module summary for details.
-
-        :rtype: `IndexedVertexList`
-        """
-        formats, initial_arrays = _parse_data(data)
-        domain = self._get_domain(True, mode, group, formats)
-
-        # Create vertex list and initialize
-        vlist = domain.create(count, len(indices))
-        start = vlist.start
-        vlist._set_index_data([i + start for i in indices])
-        for i, array in initial_arrays:
-            vlist._set_attribute_data(i, array)
-
-        return vlist
-
     def migrate(self, vertex_list, mode, group, batch):
         """Migrate a vertex list to another batch and/or group.
 
@@ -426,34 +381,33 @@ class Batch:
                 The batch to migrate to (or the current batch).
 
         """
-        formats = vertex_list.domain.__formats
+        program = vertex_list.domain.program
+        attributes = vertex_list.domain.attribute_meta
         if isinstance(vertex_list, vertexdomain.IndexedVertexList):
-            domain = batch._get_domain(True, mode, group, formats)
+            domain = batch.get_domain(True, mode, group, program, attributes)
         else:
-            domain = batch._get_domain(False, mode, group, formats)
+            domain = batch.get_domain(False, mode, group, program, attributes)
         vertex_list.migrate(domain)
 
-    def _get_domain(self, indexed, mode, group, formats):
+    def get_domain(self, indexed, mode, group, program, attributes):
         if group is None:
-            group = null_group
+            group = ShaderGroup(program=program)
 
         # Batch group
         if group not in self.group_map:
             self._add_group(group)
 
-        domain_map = self.group_map[group]
-
         # Find domain given formats, indices and mode
-        key = (formats, mode, indexed)
+        domain_map = self.group_map[group]
+        key = (indexed, mode, program, str(attributes))
         try:
             domain = domain_map[key]
         except KeyError:
             # Create domain
             if indexed:
-                domain = vertexdomain.create_indexed_domain(*formats)
+                domain = vertexdomain.IndexedVertexDomain(program, attributes)
             else:
-                domain = vertexdomain.create_domain(*formats)
-            domain.__formats = formats
+                domain = vertexdomain.VertexDomain(program, attributes)
             domain_map[key] = domain
             self._draw_list_dirty = True
 
@@ -469,6 +423,8 @@ class Batch:
             if group.parent not in self.group_children:
                 self.group_children[group.parent] = []
             self.group_children[group.parent].append(group)
+
+        group._assigned_batches.add(self)
         self._draw_list_dirty = True
 
     def _update_draw_list(self):
@@ -481,26 +437,27 @@ class Batch:
 
             # Draw domains using this group
             domain_map = self.group_map[group]
-            for (formats, mode, indexed), domain in list(domain_map.items()):
+            for (formats, mode, indexed, program_id), domain in list(domain_map.items()):
                 # Remove unused domains from batch
-                if domain._is_empty():
-                    del domain_map[(formats, mode, indexed)]
+                if domain.is_empty:
+                    del domain_map[(formats, mode, indexed, program_id)]
                     continue
-                draw_list.append(
-                    (lambda d, m: lambda: d.draw(m))(domain, mode))
+                draw_list.append((lambda d, m: lambda: d.draw(m))(domain, mode))
 
             # Sort and visit child groups of this group
             children = self.group_children.get(group)
             if children:
                 children.sort()
                 for child in list(children):
-                    draw_list.extend(visit(child))
+                    if child.visible:
+                        draw_list.extend(visit(child))
 
             if children or domain_map:
                 return [group.set_state] + draw_list + [group.unset_state]
             else:
                 # Remove unused group from batch
                 del self.group_map[group]
+                group._assigned_batches.remove(self)
                 if group.parent:
                     self.group_children[group.parent].remove(group)
                 try:
@@ -511,13 +468,15 @@ class Batch:
                     self.top_groups.remove(group)
                 except ValueError:
                     pass
+
                 return []
 
         self._draw_list = []
 
         self.top_groups.sort()
         for group in list(self.top_groups):
-            self._draw_list.extend(visit(group))
+            if group.visible:
+                self._draw_list.extend(visit(group))
 
         self._draw_list_dirty = False
 
@@ -548,8 +507,8 @@ class Batch:
             dump(group)
 
     def draw(self):
-        """Draw the batch.
-        """
+        """Draw the batch."""
+
         if self._draw_list_dirty:
             self._update_draw_list()
 
@@ -578,7 +537,7 @@ class Batch:
 
             # Draw domains using this group
             domain_map = self.group_map[group]
-            for (_, mode, _), domain in domain_map.items():
+            for (_, mode, _, _), domain in domain_map.items():
                 for alist in vertex_lists:
                     if alist.domain is domain:
                         alist.draw(mode)
@@ -588,40 +547,81 @@ class Batch:
             if children:
                 children.sort()
                 for child in children:
-                    visit(child)
+                    if child.visible:
+                        visit(child)
 
             group.unset_state()
 
         self.top_groups.sort()
         for group in self.top_groups:
-            visit(group)
+            if group.visible:
+                visit(group)
 
 
 class Group:
     """Group of common OpenGL state.
 
-    Before a vertex list is rendered, its group's OpenGL state is set; as are
-    that state's ancestors' states.  This can be defined arbitrarily on
-    subclasses; the default state change has no effect, and groups vertex
-    lists only in the order in which they are drawn.
+    Before a VertexList is rendered, its Group's OpenGL state is set.
+    This includes binding textures, shaders, or setting any other parameters.
     """
-
-    def __init__(self, parent=None):
-        """Create a group.
+    def __init__(self, order=0, parent=None):
+        """Create a Group.
 
         :Parameters:
+            `order` : int
+                Set the order to render above or below other Groups.
             `parent` : `~pyglet.graphics.Group`
-                Group to contain this group; its state will be set before this
-                state's.
+                Group to contain this Group; its state will be set before this
+                Group's state.
 
+        :Ivariables:
+            `visible` : bool
+                Determines whether this Group is visible in any of the Batches
+                it is assigned to. If False, objects in this Group will not
+                be rendered.
+            `batches` : list
+                Read Only. A list of which Batches this Group is a part of.
         """
+        self._order = order
         self.parent = parent
+        self._visible = True
+        self._assigned_batches = weakref.WeakSet()
+
+    @property
+    def order(self):
+        return self._order
+
+    @property
+    def visible(self):
+        return self._visible
+
+    @visible.setter
+    def visible(self, value):
+        self._visible = value
+
+        for batch in self._assigned_batches:
+            batch.invalidate()
+
+    @property
+    def batches(self):
+        return [batch for batch in self._assigned_batches]
 
     def __lt__(self, other):
-        return hash(self) < hash(other)
+        return self._order < other.order
+
+    def __eq__(self, other):
+        return (self.__class__ is other.__class__ and
+                self._order == other.order and
+                self.parent == other.parent)
+
+    def __hash__(self):
+        return hash((self._order, self.parent))
+
+    def __repr__(self):
+        return "{}(order={})".format(self.__class__.__name__, self._order)
 
     def set_state(self):
-        """Apply the OpenGL state change.  
+        """Apply the OpenGL state change.
         
         The default implementation does nothing."""
         pass
@@ -653,100 +653,65 @@ class Group:
             self.parent.unset_state_recursive()
 
 
-class NullGroup(Group):
-    """The default group class used when ``None`` is given to a batch.
+# Example Groups.
 
-    This implementation has no effect.
+class ShaderGroup(Group):
+    """A group that enables and binds a ShaderProgram.
     """
-    pass
 
+    def __init__(self, program, order=0, parent=None):
+        super().__init__(order, parent)
+        self.program = program
 
-#: The default group.
-#:
-#: :type: :py:class:`~pyglet.graphics.Group`
-null_group = NullGroup()
+    def set_state(self):
+        self.program.use()
+
+    def unset_state(self):
+        self.program.stop()
+
+    def __eq__(self, other):
+        return (self.__class__ is other.__class__ and
+                self._order == other.order and
+                self.program == other.program and
+                self.parent == other.parent)
+
+    def __hash__(self):
+        return hash((self._order, self.parent, self.program))
 
 
 class TextureGroup(Group):
     """A group that enables and binds a texture.
 
-    Texture groups are equal if their textures' targets and names are equal.
+    TextureGroups are equal if their textures' targets and names are equal.
     """
 
-    # Don't use this, create your own group classes that are more specific.
-    # This is just an example.
-    def __init__(self, texture, parent=None):
+    def __init__(self, texture, order=0, parent=None):
         """Create a texture group.
 
         :Parameters:
             `texture` : `~pyglet.image.Texture`
                 Texture to bind.
+            `order` : int
+                Change the order to render above or below other Groups.
             `parent` : `~pyglet.graphics.Group`
                 Parent group.
-
         """
-        super(TextureGroup, self).__init__(parent)
+        super().__init__(order, parent)
         self.texture = texture
 
     def set_state(self):
-        glEnable(self.texture.target)
+        glActiveTexture(GL_TEXTURE0)
         glBindTexture(self.texture.target, self.texture.id)
 
-    def unset_state(self):
-        glDisable(self.texture.target)
-
     def __hash__(self):
-        return hash((self.texture.target, self.texture.id, self.parent))
+        return hash((self.texture.target, self.texture.id, self.order, self.parent))
 
     def __eq__(self, other):
         return (self.__class__ is other.__class__ and
                 self.texture.target == other.texture.target and
                 self.texture.id == other.texture.id and
+                self.order == other.order and
                 self.parent == other.parent)
 
     def __repr__(self):
         return '%s(id=%d)' % (self.__class__.__name__, self.texture.id)
-
-
-class OrderedGroup(Group):
-    """A group with partial order.
-
-    Ordered groups with a common parent are rendered in ascending order of
-    their ``order`` field.  This is a useful way to render multiple layers of
-    a scene within a single batch.
-    """
-
-    # This can be useful as a top-level group, or as a superclass for other
-    # groups that need to be ordered.
-    #
-    # As a top-level group it's useful because graphics can be composited in a
-    # known order even if they don't know about each other or share any known
-    # group.
-    def __init__(self, order, parent=None):
-        """Create an ordered group.
-
-        :Parameters:
-            `order` : int
-                Order of this group.
-            `parent` : `~pyglet.graphics.Group`
-                Parent of this group.
-
-        """
-        super(OrderedGroup, self).__init__(parent)
-        self.order = order
-
-    def __lt__(self, other):
-        if isinstance(other, OrderedGroup):
-            return self.order < other.order
-        return super(OrderedGroup, self).__lt__(other)
-
-    def __eq__(self, other):
-        return (self.__class__ is other.__class__ and
-                self.order == other.order and
-                self.parent == other.parent)
-
-    def __hash__(self):
-        return hash((self.order, self.parent))
-
-    def __repr__(self):
-        return '%s(%d)' % (self.__class__.__name__, self.order)
