@@ -42,6 +42,7 @@ import sys
 
 from pyglet.event import EventDispatcher
 
+
 _is_pyglet_doc_run = hasattr(sys, "is_pyglet_doc_run") and sys.is_pyglet_doc_run
 
 
@@ -74,9 +75,11 @@ class Device:
         self.display = display
         self.name = name
         self.manufacturer = None
+        self._is_open = False
 
-        # TODO: make private 
-        self.is_open = False
+    @property
+    def is_open(self):
+        return self._is_open
 
     def open(self, window=None, exclusive=False):
         """Open the device to begin receiving input from it.
@@ -94,14 +97,14 @@ class Device:
                 opened it).
         """
 
-        if self.is_open:
+        if self._is_open:
             raise DeviceOpenException('Device is already open.')
 
-        self.is_open = True
+        self._is_open = True
 
     def close(self):
         """Close the device. """
-        self.is_open = False
+        self._is_open = False
 
     def get_controls(self):
         """Get a list of controls provided by the device.
@@ -110,8 +113,21 @@ class Device:
         """
         raise NotImplementedError('abstract')
 
+    def get_guid(self):
+        """Get the device GUID, in SDL2 format.
+
+        Return a str containing a unique device identification
+        string. This is generated from the hardware identifiers,
+        and is in the same format as was popularized by SDL2.
+        GUIDs differ between platforms, but are generally 32
+        hexidecimal characters.
+
+        :rtype: str containing the device's GUID.
+        """
+        raise NotImplementedError('abstract')
+
     def __repr__(self):
-        return '%s(name=%s)' % (self.__class__.__name__, self.name)
+        return f"{self.__class__.__name__}(name={self.name})"
 
 
 class Control(EventDispatcher):
@@ -135,10 +151,10 @@ class Control(EventDispatcher):
             operating systems.
     """
 
-    def __init__(self, name, raw_name=None):
+    def __init__(self, name, raw_name=None, inverted=False):
         self.name = name
         self.raw_name = raw_name
-        self.inverted = False
+        self.inverted = inverted
         self._value = None
 
     @property
@@ -162,21 +178,20 @@ class Control(EventDispatcher):
 
     def __repr__(self):
         if self.name:
-            return '%s(name=%s, raw_name=%s)' % (
-                self.__class__.__name__, self.name, self.raw_name)
+            return f"{self.__class__.__name__}(name={self.name}, raw_name={self.raw_name})"
         else:
-            return '%s(raw_name=%s)' % (self.__class__.__name__, self.raw_name)
+            return f"{self.__class__.__name__}(raw_name={self.raw_name})"
 
-    if _is_pyglet_doc_run:
-        def on_change(self, value):
-            """The value changed.
+    def on_change(self, value):
+        """The value changed.
 
-            :Parameters:
-                `value` : float
-                    Current value of the control.
+        :Parameters:
+            `value` : float
+                Current value of the control.
+
+        :event:
+        """
 
-            :event:
-            """
 
 Control.register_event_type('on_change')
 
@@ -206,20 +221,20 @@ class RelativeAxis(Control):
         return self._value
 
     @value.setter
-    def value(self, newvalue):
-        self._value = newvalue
-        self.dispatch_event('on_change', newvalue)
+    def value(self, value):
+        self._value = value
+        self.dispatch_event('on_change', value)
 
 
 class AbsoluteAxis(Control):
     """An axis whose value represents a physical measurement from the device.
 
-    The value is advertised to range over ``min`` and ``max``.
+    The value is advertised to range over ``minimum`` and ``maximum``.
 
     :Ivariables:
-        `min` : float
+        `minimum` : float
             Minimum advertised value.
-        `max` : float 
+        `maximum` : float
             Maximum advertised value.
     """
 
@@ -245,11 +260,10 @@ class AbsoluteAxis(Control):
     #: described by two orthogonal controls.
     HAT_Y = 'hat_y'
 
-    def __init__(self, name, min, max, raw_name=None):
-        super(AbsoluteAxis, self).__init__(name, raw_name)
-        
-        self.min = min
-        self.max = max
+    def __init__(self, name, minimum, maximum, raw_name=None, inverted=False):
+        super().__init__(name, raw_name, inverted)
+        self.min = minimum
+        self.max = maximum
 
 
 class Button(Control):
@@ -283,15 +297,20 @@ class Button(Control):
             :event:
             """
 
+
 Button.register_event_type('on_press')
 Button.register_event_type('on_release')
 
 
 class Joystick(EventDispatcher):
-    """High-level interface for joystick-like devices.  This includes analogue
-    and digital joysticks, gamepads, game controllers, and possibly even
-    steering wheels and other input devices.  There is unfortunately no way to
-    distinguish between these different device types.
+    """High-level interface for joystick-like devices.  This includes a wide range
+    of analog and digital joysticks, gamepads, controllers, and possibly even
+    steering wheels and other input devices. There is unfortunately no easy way to
+    distinguish between most of these different device types.
+
+    For a simplified subset of Joysticks, see the :py:class:`~pyglet.input.Controller`
+    interface. This covers a variety of popular game console controllers. Unlike
+    Joysticks, Controllers have strictly defined layouts and inputs.
 
     To use a joystick, first call `open`, then in your game loop examine
     the values of `x`, `y`, and so on.  These values are normalized to the
@@ -326,7 +345,7 @@ class Joystick(EventDispatcher):
             (bottom).
         `z` : float
             Current Z value ranging from -1.0 to 1.0.  On joysticks the Z
-            value is usually the throttle control.  On game controllers the Z
+            value is usually the throttle control.  On controllers the Z
             value is usually the secondary thumb vertical axis.
         `rx` : float
             Current rotational X value ranging from -1.0 to 1.0.
@@ -511,12 +530,371 @@ class Joystick(EventDispatcher):
                 (centered) or 1 (top).
         """
 
+
 Joystick.register_event_type('on_joyaxis_motion')
 Joystick.register_event_type('on_joybutton_press')
 Joystick.register_event_type('on_joybutton_release')
 Joystick.register_event_type('on_joyhat_motion')
 
 
+class Controller(EventDispatcher):
+
+    __slots__ = ('device', 'guid', '_mapping', 'name', 'a', 'b', 'x', 'y',
+                 'back', 'start', 'guide', 'leftshoulder', 'rightshoulder',
+                 'leftstick', 'rightstick', 'lefttrigger', 'righttrigger',
+                 'leftx', 'lefty', 'rightx', 'righty', 'dpup', 'dpdown', 'dpleft',
+                 'dpright', '_button_controls', '_axis_controls', '_hat_control',
+                 '_hat_x_control', '_hat_y_control')
+
+    def __init__(self, device, mapping):
+        """High-level interface for Game Controllers.
+
+        Unlike Joysticks, Controllers have a strictly defined set of inputs
+        that matches the layout of popular home video game console Controllers.
+        This includes a variety of face and shoulder buttons, analog sticks and
+        triggers, a directional pad, and optional rumble (force feedback)
+        effects.
+
+        To use a Controller, you must first call `open`. Controllers will then
+        dispatch a variety of events whenever the inputs change. They can also
+        be polled at any time to find the current value of any inputs. Analog
+        inputs are normalized to the range [-1.0, 1.0].
+
+        :note: A running application event loop is required
+
+        The following event types are dispatched:
+            `on_button_press`
+            `on_button_release`
+            `on_stick_motion`
+            `on_dpad_motion`
+            `on_trigger_motion`
+
+        The device name can be queried to get the name of the joystick.
+
+        :Ivariables:
+            `device` : `Device`
+                The underlying device used by this joystick interface.
+            `name` : str
+                The name of the Controller as reported by the OS.
+            `guid` : str
+                The unique device identification string, in SDL2 format.
+            `a` : bool
+            `b` : bool
+            `x` : bool
+            `x` : bool
+            `back` : bool
+            `start` : bool
+            `guide` : bool
+            `leftshoulder` : bool
+            `rightshoulder` : bool
+            `leftstick` : bool
+            `rightstick` : bool
+            `leftx` : float
+            `lefty` : float
+            `rightx` : float
+            `righty` : float
+            `lefttrigger` : float
+            `righttrigger` : float
+            `dpup` : bool
+            `dpdown` : bool
+            `dpleft` : bool
+            `dpright` : bool
+
+        .. versionadded:: 2.0
+        """
+
+        self.device = device
+        self._mapping = mapping
+
+        self.name = mapping.get('name')
+        self.guid = mapping.get('guid')
+
+        self.a = False
+        self.b = False
+        self.x = False
+        self.y = False
+        self.back = False
+        self.start = False
+        self.guide = False
+        self.leftshoulder = False
+        self.rightshoulder = False
+        self.leftstick = False          # stick press button
+        self.rightstick = False         # stick press button
+        self.lefttrigger = 0
+        self.righttrigger = 0
+        self.leftx = 0
+        self.lefty = 0
+        self.rightx = 0
+        self.righty = 0
+        self.dpup = False
+        self.dpdown = False
+        self.dpleft = False
+        self.dpright = False
+
+        self._button_controls = []
+        self._axis_controls = []
+        self._hat_control = None
+        self._hat_x_control = None
+        self._hat_y_control = None
+
+        self._initialize_controls()
+
+    def _initialize_controls(self):
+
+        def add_axis(control, axis_name):
+            tscale = 1.0 / (control.max - control.min)
+            scale = 2.0 / (control.max - control.min)
+            bias = -1.0 - control.min * scale
+            if control.inverted:
+                scale = -scale
+                bias = -bias
+
+            if axis_name in ("dpup", "dpdown"):
+                @control.event
+                def on_change(value):
+                    normalized_value = value * scale + bias
+                    self.dpup = self.dpdown = False
+                    if normalized_value > 0.1:
+                        self.dpup = True
+                    if normalized_value < -0.1:
+                        self.dpdown = True
+                    self.dispatch_event('on_dpad_motion', self,
+                                        self.dpleft, self.dpright, self.dpup, self.dpdown)
+
+            elif axis_name in ("dpleft", "dpright"):
+                @control.event
+                def on_change(value):
+                    normalized_value = value * scale + bias
+                    self.dpleft = self.dpright = False
+                    if normalized_value > 0.1:
+                        self.dpright = True
+                    if normalized_value < -0.1:
+                        self.dpleft = True
+                    self.dispatch_event('on_dpad_motion', self, self.dpleft, self.dpright, self.dpup, self.dpdown)
+
+            elif axis_name in ("lefttrigger", "righttrigger"):
+                @control.event
+                def on_change(value):
+                    normalized_value = value * tscale
+                    setattr(self, axis_name, normalized_value)
+                    self.dispatch_event('on_trigger_motion', self, axis_name, normalized_value)
+
+            elif axis_name in ("leftx", "lefty"):
+                @control.event
+                def on_change(value):
+                    normalized_value = value * scale + bias
+                    setattr(self, axis_name, normalized_value)
+                    self.dispatch_event('on_stick_motion', self,
+                                        "leftstick", self.leftx, -self.lefty)
+
+            elif axis_name in ("rightx", "righty"):
+                @control.event
+                def on_change(value):
+                    normalized_value = value * scale + bias
+                    setattr(self, axis_name, normalized_value)
+                    self.dispatch_event('on_stick_motion', self,
+                                        "rightstick", self.rightx, -self.righty)
+
+        def add_button(control, button_name):
+            if button_name in ("dpleft", "dpright", "dpup", "dpdown"):
+                @control.event
+                def on_change(value):
+                    setattr(self, button_name, value)
+                    self.dispatch_event('on_dpad_motion', self,
+                                        self.dpleft, self.dpright, self.dpup, self.dpdown)
+            else:
+                @control.event
+                def on_change(value):
+                    setattr(self, button_name, value)
+
+                @control.event
+                def on_press():
+                    self.dispatch_event('on_button_press', self, button_name)
+
+                @control.event
+                def on_release():
+                    self.dispatch_event('on_button_release', self, button_name)
+
+        def add_dedicated_hat(control):
+            # 8-directional hat encoded as a single control (Windows/Mac)
+            @control.event
+            def on_change(value):
+                if value & 0xffff == 0xffff:
+                    self.dpleft = self.dpright = self.dpup = self.dpdown = False
+                else:
+                    if control.max > 8:  # DirectInput: scale value
+                        value //= 0xfff
+                    if 0 <= value < 8:
+                        self.dpleft, self.dpright, self.dpup, self.dpdown = (
+                            (False, False, True,  False),
+                            (False, True,  True,  False),
+                            (False, True,  False, False),
+                            (False, True,  False, True),
+                            (False, False, False, True),
+                            (True,  False, False, True),
+                            (True,  False, False, False),
+                            (True,  False, True,  False))[value]
+                    else:
+                        # Out of range
+                        self.dpleft = self.dpright = self.dpup = self.dpdown = False
+                self.dispatch_event('on_dpad_motion', self,
+                                    self.dpleft, self.dpright, self.dpup, self.dpdown)
+
+        for control in self.device.get_controls():
+            """Categorize the various control types"""
+            if isinstance(control, Button):
+                self._button_controls.append(control)
+
+            elif isinstance(control, AbsoluteAxis):
+                if control.name in ('x', 'y', 'z', 'rx', 'ry', 'rz'):
+                    self._axis_controls.append(control)
+                elif control.name == "hat_x":
+                    self._hat_x_control = control
+                elif control.name == "hat_y":
+                    self._hat_y_control = control
+                elif control.name == "hat":
+                    self._hat_control = control
+
+        for name, relation in self._mapping.items():
+
+            if relation is None or type(relation) is str:
+                continue
+
+            if relation.control_type == "button":
+                try:
+                    add_button(self._button_controls[relation.index], name)
+                except IndexError:
+                    continue
+            elif relation.control_type == "axis":
+                try:
+                    add_axis(self._axis_controls[relation.index], name)
+                except IndexError:
+                    continue
+            elif relation.control_type == "hat0":
+                if self._hat_control:
+                    # TODO: test this on Windows/Mac.
+                    add_dedicated_hat(self._hat_control)
+                else:
+                    if relation.index == 1:       # 1 == UP
+                        add_axis(self._hat_y_control, "dpup")
+                    elif relation.index == 2:     # 2 == RIGHT
+                        add_axis(self._hat_x_control, "dpright")
+                    elif relation.index == 4:     # 4 == DOWN
+                        add_axis(self._hat_y_control, "dpdown")
+                    elif relation.index == 8:     # 8 == LEFT
+                        add_axis(self._hat_x_control, "dpleft")
+
+    def open(self, window=None, exclusive=False):
+        """Open the controller.  See `Device.open`. """
+        self.device.open(window, exclusive)
+
+    def close(self):
+        """Close the controller.  See `Device.close`. """
+        self.device.close()
+
+    # Rumble (force feedback) methods:
+
+    def rumble_play_weak(self, strength=1.0, duration=0.5):
+        """Play a rumble effect on the weak motor.
+
+        :Parameters:
+            `strength` : float
+                The strength of the effect, from 0 to 1.
+            `duration` : float
+                The duration of the effect in seconds.
+        """
+
+    def rumble_play_strong(self, strength=1.0, duration=0.5):
+        """Play a rumble effect on the strong motor.
+
+        :Parameters:
+            `strength` : float
+                The strength of the effect, from 0 to 1.
+            `duration` : float
+                The duration of the effect in seconds.
+        """
+
+    def rumble_stop_weak(self):
+        """Stop playing rumble effects on the weak motor."""
+
+    def rumble_stop_strong(self):
+        """Stop playing rumble effects on the strong motor."""
+
+    # Input Event types:
+
+    def on_stick_motion(self, controller, stick, xvalue, yvalue):
+        """The value of a controller analogue stick changed.
+
+        :Parameters:
+            `controller` : `Controller`
+                The controller whose analogue stick changed.
+            `stick` : string
+                The name of the stick that changed.
+            `xvalue` : float
+                The current X axis value, normalized to [-1, 1].
+            `yvalue` : float
+                The current Y axis value, normalized to [-1, 1].
+        """
+
+    def on_dpad_motion(self, controller, dpleft, dpright, dpup, dpdown):
+        """The direction pad of the controller changed.
+
+        :Parameters:
+            `controller` : `Controller`
+                The controller whose hat control changed.
+            `dpleft` : boolean
+                True if left is pressed on the directional pad.
+            `dpright` : boolean
+                True if right is pressed on the directional pad.
+            `dpup` : boolean
+                True if up is pressed on the directional pad.
+            `dpdown` : boolean
+                True if down is pressed on the directional pad.
+        """
+
+    def on_trigger_motion(self, controller, trigger, value):
+        """The value of a controller analogue stick changed.
+
+        :Parameters:
+            `controller` : `Controller`
+                The controller whose analogue stick changed.
+            `trigger` : string
+                The name of the trigger that changed.
+            `value` : float
+                The current value of the trigger, normalized to [-1, 1].
+        """
+
+    def on_button_press(self, controller, button):
+        """A button on the controller was pressed.
+
+        :Parameters:
+            `controller` :  :py:class:`Controller`
+                The controller whose button was pressed.
+            `button` : string
+                The name of the button that was pressed.
+        """
+
+    def on_button_release(self, controller, button):
+        """A button on the joystick was released.
+
+        :Parameters:
+            `controller` : `Controller`
+                The controller whose button was released.
+            `button` : string
+                The name of the button that was released.
+        """
+
+    def __repr__(self):
+        return f"Controller(name={self.name})"
+
+
+Controller.register_event_type('on_button_press')
+Controller.register_event_type('on_button_release')
+Controller.register_event_type('on_stick_motion')
+Controller.register_event_type('on_dpad_motion')
+Controller.register_event_type('on_trigger_motion')
+
+
 class AppleRemote(EventDispatcher):
     """High-level interface for Apple remote control.
 
@@ -597,10 +975,10 @@ class AppleRemote(EventDispatcher):
 
         The 'select_hold' and 'menu_hold' button release events are sent
         immediately after the corresponding press events regardless of
-        whether or not the user has released the button.
+        whether the user has released the button.
 
         :Parameters:
-            `button` : unicode
+            `button` : str
                 The name of the button that was released. The valid names are
                 'up', 'down', 'left', 'right', 'left_hold', 'right_hold',
                 'menu', 'menu_hold', 'select', and 'select_hold'
@@ -608,6 +986,7 @@ class AppleRemote(EventDispatcher):
         :event:
         """
 
+
 AppleRemote.register_event_type('on_button_press')
 AppleRemote.register_event_type('on_button_release')
 
@@ -698,7 +1077,7 @@ class TabletCanvas(EventDispatcher):
             :event:
             """
 
-        def on_motion(self, cursor, x, y, pressure, tilt_x, tilt_y):
+        def on_motion(self, cursor, x, y, pressure, tilt_x, tilt_y, buttons):
             """The cursor moved on the tablet surface.
 
             If `pressure` is 0, then the cursor is actually hovering above the
@@ -718,10 +1097,14 @@ class TabletCanvas(EventDispatcher):
                     Currently undefined.
                 `tilt_y` : float
                     Currently undefined.
+                `buttons` : int
+                    Button state may be provided if the platform supports it.
+                    Supported on: Windows
 
             :event:
             """
 
+
 TabletCanvas.register_event_type('on_enter')
 TabletCanvas.register_event_type('on_leave')
 TabletCanvas.register_event_type('on_motion')
@@ -745,3 +1128,74 @@ class TabletCursor:
 
     def __repr__(self):
         return '%s(%s)' % (self.__class__.__name__, self.name)
+
+
+class ControllerManager(EventDispatcher):
+    """High level interface for managing game Controllers.
+
+    This class provides a convenient way to handle the
+    connection and disconnection of devices. A list of all
+    connected Controllers can be queried at any time with the
+    `get_controllers` method. For hot-plugging, events are
+    dispatched for `on_connect` and `on_disconnect`.
+    To use the ControllerManager, first make an instance::
+
+        controller_man = pyglet.input.ControllerManager()
+
+    At the start of your game, query for any Controllers
+    that are already connected::
+
+        controllers = controller_man.get_controllers()
+
+    To handle Controllers that are connected or disconnected
+    after the start of your game, register handlers for the
+    appropriate events::
+
+        @controller_man.event
+        def on_connect(controller):
+            # code to handle newly connected
+            # (or re-connected) controllers
+            controller.open()
+            print("Connect:", controller)
+
+        @controller_man.event
+        def on_disconnect(controller):
+            # code to handle disconnected Controller
+            print("Disconnect:", controller)
+
+    .. versionadded:: 1.2
+    """
+
+    def get_controllers(self):
+        """Get a list of all connected Controllers
+
+        :rtype: list of :py:class:`Controller`
+        """
+        raise NotImplementedError
+
+    def on_connect(self, controller):
+        """A Controller has been connected. If this is
+        a previously dissconnected Controller that is
+        being re-connected, the same Controller instance
+        will be returned.
+
+        :Parameters:
+            `controller` : :py:class:`Controller`
+                An un-opened Controller instance.
+
+        :event:
+        """
+
+    def on_disconnect(self, controller):
+        """A Controller has been disconnected.
+
+        :Parameters:
+            `controller` : :py:class:`Controller`
+                An un-opened Controller instance.
+
+        :event:
+        """
+
+
+ControllerManager.register_event_type('on_connect')
+ControllerManager.register_event_type('on_disconnect')
