@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2021 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -75,28 +75,16 @@ the application performance can be delayed.
 The player provides a :py:meth:`Player.delete` method that can be used to
 release resources immediately.
 """
-
 from .drivers import get_audio_driver
-from .exceptions import MediaDecodeException
 from .player import Player, PlayerGroup
-from .codecs import get_decoders, get_encoders, add_decoders, add_encoders
-from .codecs import add_default_media_codecs, have_ffmpeg
-from .codecs import Source, StaticSource, StreamingSource, SourceGroup
+from .codecs import registry as _codec_registry
+from .codecs import add_default_codecs as _add_default_codecs
+from .codecs import Source, StaticSource, StreamingSource, SourceGroup, have_ffmpeg
 
 from . import synthesis
 
 
-__all__ = (
-    'load',
-    'get_audio_driver',
-    'Player',
-    'PlayerGroup',
-    'SourceGroup',
-    'get_encoders',
-    'get_decoders',
-    'add_encoders',
-    'add_decoders',
-)
+__all__ = 'load', 'get_audio_driver', 'Player', 'PlayerGroup', 'SourceGroup', 'StaticSource', 'StreamingSource'
 
 
 def load(filename, file=None, streaming=True, decoder=None):
@@ -122,22 +110,9 @@ def load(filename, file=None, streaming=True, decoder=None):
     :rtype: StreamingSource or Source
     """
     if decoder:
-        return decoder.decode(file, filename, streaming)
+        return decoder.decode(filename, file, streaming=streaming)
     else:
-        first_exception = None
-        for decoder in get_decoders(filename):
-            try:
-                loaded_source = decoder.decode(file, filename, streaming)
-                return loaded_source
-            except MediaDecodeException as e:
-                if not first_exception or first_exception.exception_priority < e.exception_priority:
-                    first_exception = e
-
-        # TODO: Review this:
-        # The FFmpeg codec attempts to decode anything, so this codepath won't be reached.
-        if not first_exception:
-            raise MediaDecodeException('No decoders are available for this media format.')
-        raise first_exception
+        return _codec_registry.decode(filename, file, streaming=streaming)
 
 
-add_default_media_codecs()
+_add_default_codecs()
