@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2021 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,14 +33,13 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-import ctypes
 import math
+import ctypes
 
-import pyglet
 from . import interface
 from pyglet.util import debug_print
-from pyglet.media.events import MediaEvent
-from pyglet.media.drivers.base import AbstractAudioDriver, AbstractAudioPlayer
+from pyglet.media.mediathreads import PlayerWorkerThread
+from pyglet.media.drivers.base import AbstractAudioDriver, AbstractAudioPlayer, MediaEvent
 from pyglet.media.drivers.listener import AbstractListener
 
 _debug = debug_print('debug_media')
@@ -92,7 +91,7 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
 
         # Up to one audio data may be buffered if too much data was received
         # from the source that could not be written immediately into the
-        # buffer.  See refill().
+        # buffer.  See _refill().
         self._audiodata_buffer = None
 
         # Theoretical write and play cursors for an infinite buffer.  play
@@ -106,7 +105,7 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
         self._eos_cursor = None
 
         # Indexes into DSound circular buffer.  Complications ensue wrt each
-        # other to avoid writing over the play cursor.  See get_write_size and
+        # other to avoid writing over the play cursor.  See _get_write_size and
         # write().
         self._play_cursor_ring = 0
         self._write_cursor_ring = 0
@@ -126,18 +125,18 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
 
         self._ds_buffer.current_position = 0
 
-        self.refill(self._buffer_size)
+        self._refill(self._buffer_size)
 
     def __del__(self):
         # We decrease the IDirectSound refcount
         self.driver._ds_driver._native_dsound.Release()
 
     def delete(self):
-        pyglet.clock.unschedule(self._check_refill)
+        self.driver.worker.remove(self)
 
     def play(self):
         assert _debug('DirectSound play')
-        pyglet.clock.schedule_interval(self._check_refill, 0.1)
+        self.driver.worker.add(self)
 
         if not self._playing:
             self._get_audiodata()  # prebuffer if needed
@@ -148,7 +147,7 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
 
     def stop(self):
         assert _debug('DirectSound stop')
-        pyglet.clock.unschedule(self._check_refill)
+        self.driver.worker.remove(self)
 
         if self._playing:
             self._playing = False
@@ -167,14 +166,16 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
         del self._events[:]
         del self._timestamps[:]
 
-    def _check_refill(self, dt):
-        write_size = self.get_write_size()
+    def refill_buffer(self):
+        write_size = self._get_write_size()
         if write_size > self.min_buffer_size:
-            self.refill(write_size)
+            self._refill(write_size)
+            return True
+        return False
 
-    def refill(self, write_size):
+    def _refill(self, write_size):
         while write_size > 0:
-            assert _debug('refill, write_size =', write_size)
+            assert _debug('_refill, write_size =', write_size)
             audio_data = self._get_audiodata()
 
             if audio_data is not None:
@@ -192,7 +193,7 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
                 and self._play_cursor > self._eos_cursor)
 
     def _dispatch_new_event(self, event_name):
-        MediaEvent(0, event_name)._sync_dispatch_to_player(self.player)
+        MediaEvent(event_name).sync_dispatch_to_player(self.player)
 
     def _get_audiodata(self):
         if self._audiodata_buffer is None or self._audiodata_buffer.length == 0:
@@ -282,7 +283,7 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
             self.stop()
             self._dispatch_new_event('on_eos')
 
-    def get_write_size(self):
+    def _get_write_size(self):
         self.update_play_cursor()
 
         play_cursor = self._play_cursor
@@ -373,8 +374,8 @@ class DirectSoundAudioPlayer(AbstractAudioPlayer):
             self._ds_buffer.cone_outside_volume = volume
 
     def prefill_audio(self):
-        write_size = self.get_write_size()
-        self.refill(write_size)
+        write_size = self._get_write_size()
+        self._refill(write_size)
 
 
 class DirectSoundDriver(AbstractAudioDriver):
@@ -385,6 +386,9 @@ class DirectSoundDriver(AbstractAudioDriver):
         assert self._ds_driver is not None
         assert self._ds_listener is not None
 
+        self.worker = PlayerWorkerThread()
+        self.worker.start()
+
     def __del__(self):
         self.delete()
 
@@ -402,6 +406,8 @@ class DirectSoundDriver(AbstractAudioDriver):
         return DirectSoundListener(self._ds_listener, self._ds_driver.primary_buffer)
 
     def delete(self):
+        if hasattr(self, 'worker'):
+            self.worker.stop()
         # Make sure the _ds_listener is deleted before the _ds_driver
         self._ds_listener = None
 
