@@ -87,7 +87,7 @@ import pyglet
 from pyglet.gl import GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
 from pyglet.gl import GL_TRIANGLES, GL_LINES, GL_BLEND
 from pyglet.gl import glBlendFunc, glEnable, glDisable
-from pyglet.graphics import shader, Batch, Group
+from pyglet.graphics import Batch, Group
 
 
 vertex_source = """#version 150 core
