@@ -64,9 +64,11 @@ Usage::
 from pyglet import compat_platform
 
 
-class KeyStateHandler(dict):
+class KeyStateHandler:
     """Simple handler that tracks the state of keys on the keyboard. If a
     key is pressed then this handler holds a True value for it.
+    If the window loses focus, all keys will be reset to False to avoid a
+    "sticky" key state.
 
     For example::
 
@@ -82,14 +84,20 @@ class KeyStateHandler(dict):
         False
 
     """
+    def __init__(self):
+        self.data = {}
+
     def on_key_press(self, symbol, modifiers):
-        self[symbol] = True
+        self.data[symbol] = True
 
     def on_key_release(self, symbol, modifiers):
-        self[symbol] = False
+        self.data[symbol] = False
+
+    def on_deactivate(self):
+        self.data.clear()
 
     def __getitem__(self, key):
-        return self.get(key, False)
+        return self.data.get(key, False)
 
 
 def modifiers_string(modifiers):
@@ -255,6 +263,8 @@ MOTION_BEGINNING_OF_FILE = 5
 MOTION_END_OF_FILE       = 6
 MOTION_BACKSPACE         = BACKSPACE
 MOTION_DELETE            = DELETE
+MOTION_COPY              = 7
+MOTION_PASTE             = 8
 
 # Number pad
 NUMLOCK       = 0xff7f
