@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2021 pyglet contributors
+# Copyright (c) 2008-2022 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -37,9 +37,11 @@
 """
 
 
-class MouseStateHandler(dict):
+class MouseStateHandler:
     """Simple handler that tracks the state of buttons from the mouse. If a
     button is pressed then this handler holds a True value for it.
+    If the window loses focus, all buttons will be reset to False in order
+    to avoid a "sticky" button state.
 
     For example::
 
@@ -55,14 +57,32 @@ class MouseStateHandler(dict):
         False
 
     """
+
+    def __init__(self):
+        self.data = {
+            "x": 0,
+            "y": 0,
+        }
+
     def on_mouse_press(self, x, y, button, modifiers):
-        self[button] = True
-        
+        self.data[button] = True
+
     def on_mouse_release(self, x, y, button, modifiers):
-        self[button] = False
-        
+        self.data[button] = False
+
+    def on_deactivate(self):
+        self.data.clear()
+
+    def on_mouse_motion(self, x, y, dx, dy):
+        self.data["x"] = x
+        self.data["y"] = y
+
+    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
+        self.data["x"] = x
+        self.data["y"] = y
+
     def __getitem__(self, key):
-        return self.get(key, False)
+        return self.data.get(key, False)
 
 
 def buttons_string(buttons):
@@ -81,15 +101,21 @@ def buttons_string(buttons):
     """
     button_names = []
     if buttons & LEFT:
-        button_names.append('LEFT')
+        button_names.append("LEFT")
     if buttons & MIDDLE:
-        button_names.append('MIDDLE')
+        button_names.append("MIDDLE")
     if buttons & RIGHT:
-        button_names.append('RIGHT')
-    return '|'.join(button_names)
+        button_names.append("RIGHT")
+    if buttons & MOUSE4:
+        button_names.append("MOUSE4")
+    if buttons & MOUSE5:
+        button_names.append("MOUSE5")
+    return "|".join(button_names)
 
 
 # Symbolic names for the mouse buttons
 LEFT = 1 << 0
 MIDDLE = 1 << 1
 RIGHT = 1 << 2
+MOUSE4 = 1 << 3
+MOUSE5 = 1 << 4
